"""group_subscriptions

Revision: 0092
Revision ID: 1f5aaa83bebb
Revises: 10fc44ad31fa
Create Date: 2017-04-11 10:57:02.945124

"""

# revision identifiers, used by Alembic.
revision = '1f5aaa83bebb'
down_revision = '10fc44ad31fa'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('card_group_subscriptions',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('group_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('auto', sa.Boolean(), nullable=True),
        sa.Column('send_emails', sa.Boolean(), nullable=True),
        sa.Column('notify_card_changes', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['group_id'], ['card_groups.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_group_subscriptions_group_id'), 'card_group_subscriptions', ['group_id'], unique=False)
    op.create_index(op.f('ix_card_group_subscriptions_group_id_user_id'), 'card_group_subscriptions', ['group_id', 'user_id'], unique=False)


def downgrade():
    op.drop_table('card_group_subscriptions')



"""attribute_containers

Revision: 0080
Revision ID: 30ad9d5dfff6
Revises: 369797b75a28
Create Date: 2016-12-22 11:08:56.517631

"""

# revision identifiers, used by Alembic.
revision = '30ad9d5dfff6'
down_revision = '369797b75a28'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.add_column('attribute_groups', sa.Column('containers', postgresql.JSONB(), nullable=True))
    op.execute("""UPDATE attributes SET container = '__top__' WHERE container = 'top'""")
    op.execute("""UPDATE attribute_positions SET container = '__top__' WHERE container = 'top'""")


def downgrade():
    op.drop_column('attribute_groups', 'containers')



"""added cache_id

Revision: 0006
Revision ID: 43064fa5bc36
Revises: 22bacef12338
Create Date: 2015-06-10 13:53:29.129453

"""

# revision identifiers, used by Alembic.
revision = '43064fa5bc36'
down_revision = '22bacef12338'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('organizations', sa.Column('cache_id', sa.String(), nullable=True))


def downgrade():
    op.drop_column('organizations', 'cache_id')



"""global_views_options_ref

Revision: 0194
Revision ID: 7230cc1e84bc
Revises: 60fef5bde116
Create Date: 2020-09-22 11:31:48.293106

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '7230cc1e84bc'
down_revision = '60fef5bde116'
branch_labels = None
depends_on = None


views = sa.Table(
    'views',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer),
    sa.Column('user_id', sa.Integer),
    sa.Column('filter', sa.String),
    sa.Column('mode', sa.String),
    sa.Column('mode_options', postgresql.JSONB)
)


def upgrade():
    op.execute("""
        UPDATE views
        SET filter_scopes = jsonb_build_array(jsonb_build_array('user', user_id))
        WHERE project_id IS NULL AND mode != 'report'
    """)

    conn = op.get_bind()
    updates = []
    for view in conn.execute(views.select().where(sa.and_(views.c.mode == 'report', views.c.project_id==None))):
        if isinstance(view['mode_options'], dict) and view['mode_options'].get('widgets'):
            for widget in view['mode_options']['widgets']:
                widget['filter'] = view['filter']
                widget['filter_scopes'] = [['user', view['user_id']]]
            updates.append({'vid': view['id'], 'opts': view['mode_options']})

    if updates:
        stmt = views.update().where(views.c.id==sa.bindparam('vid')).values(
            mode_options=sa.bindparam('opts'))
        conn.execute(stmt, updates)

def downgrade():
    pass



"""cleanup_orphan_views

Revision ID: 0246
Revises: 0245
Create Date: 2022-10-13 17:40:33.206604

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0246'
down_revision = '0245'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        DELETE FROM views WHERE project_id IS NOT NULL
        AND card_ref NOT IN (SELECT ref FROM cards WHERE cards.project_id = views.project_id AND (parent_id IS NULL OR board_created))
    """)


def downgrade():
    pass



"""form-views

Revision: 0210
Revision ID: ee6fba69aef3
Revises: f8fc6ca825a2
Create Date: 2021-05-24 16:16:10.321086

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'ee6fba69aef3'
down_revision = 'f8fc6ca825a2'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        DELETE FROM views WHERE mode = 'forms';
    """)

    op.execute("""
        INSERT INTO views (created_at, created_by, mode, mode_options, name, project_id, card_ref, limit_to_ref, position)
        SELECT
            rf.created_at,
            rf.created_by_id,
            'form',
            jsonb_build_object('formId', rf.id),
            rf.name,
            rf.project_id,
            0,
            true,
            (SELECT MAX(position) FROM views WHERE project_id = rf.project_id)
        FROM request_forms rf
    """)


def downgrade():
    pass



"""two_factor_auth

Revision: 0165
Revision ID: f5482137635
Revises: 13a7a762d14a
Create Date: 2019-04-11 16:45:34.794811

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f5482137635'
down_revision = '13a7a762d14a'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('two_factor_auth_enabled', sa.Boolean(), nullable=True))
    op.add_column('users', sa.Column('otp_code', sa.String(), nullable=True))
    op.add_column('users', sa.Column('otp_recovery_code', sa.String(), nullable=True))
    op.create_index(op.f('ix_users_otp_code'), 'users', ['otp_code'], unique=True)
    op.execute("UPDATE users SET two_factor_auth_enabled = false")


def downgrade():
    op.drop_column('users', 'two_factor_auth_enabled')
    op.drop_column('users', 'otp_code')
    op.drop_column('users', 'otp_recovery_code')



"""user_logins

Revision: 0099
Revision ID: 439d72ec8dc4
Revises: 3a512b4f3d8f
Create Date: 2017-06-30 15:22:35.192111

"""

# revision identifiers, used by Alembic.
revision = '439d72ec8dc4'
down_revision = '3a512b4f3d8f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    user_logins = op.create_table('user_logins',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('login_at', sa.DateTime(), nullable=True),
        sa.Column('login_from', sa.String(), nullable=True),
        sa.Column('login_provider', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_user_logins_user_id'), 'user_logins', ['user_id'], unique=False)


def downgrade():
    op.drop_table('user_logins')



# -*- coding: utf-8 -*-
"""dashboard_sections

Revision: 0085
Revision ID: 2d68c5dd350e
Revises: 427a0e562031
Create Date: 2017-02-27 13:26:37.792586

"""

# revision identifiers, used by Alembic.
revision = '2d68c5dd350e'
down_revision = '427a0e562031'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.add_column('users', sa.Column('dashboard_sections', postgresql.JSON(), nullable=True))

    op.execute("""
        UPDATE users SET dashboard_sections = ('['
            || '{"title": "Overdue cards assigned to me", "filter": "@me {due date}<today state!=done", "collapsed": false},'
            || '{"title": "Due cards assigned to me", "filter": "@me {due date}>=today state!=done", "collapsed": false},'
            || '{"title": "Assigned to me", "filter": "@me {due date}=empty state!=done", "collapsed": false}]')::json
        WHERE locale is null or locale != 'fr'
    """)

    op.execute("""
        UPDATE users SET dashboard_sections = ('['
            || '{"title": "Cartes en retard qui me sont assignées", "filter": "@me {due date}<today state!=done", "collapsed": false},'
            || '{"title": "Cartes avec une échéance qui me sont assignées", "filter": "@me {due date}>=today state!=done", "collapsed": false},'
            || '{"title": "Cartes qui me sont assignées", "filter": "@me {due date}=empty state!=done", "collapsed": false}]')::json
        WHERE locale = 'fr'
    """)


def downgrade():
    op.drop_column('users', 'dashboard_sections')



"""team_icon

Revision: 0209
Revision ID: f8fc6ca825a2
Revises: 41d379ba09af
Create Date: 2021-05-05 16:06:07.183238

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f8fc6ca825a2'
down_revision = '41d379ba09af'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('teams', sa.Column('icon', sa.String(), nullable=True))


def downgrade():
    op.drop_column('teams', 'icon')



"""cleanup_orphan_card_logs

Revision ID: 0250
Revises: 0249
Create Date: 2023-05-05 21:10:33.820171

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0250'
down_revision = '0249'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        DELETE FROM card_logs
        WHERE id IN (
            SELECT cl.id
            FROM card_logs cl
            JOIN card_log_types clt ON clt.id = cl.log_type_id
            JOIN project_enabled_log_types pelt ON pelt.log_type_id = clt.id
            JOIN cards c ON c.id = cl.card_id
            WHERE pelt.project_id != c.project_id
        )
    """)


def downgrade():
    pass



"""stream_items

Revision: 0066
Revision ID: 16627338400e
Revises: 2daf3bc83d75
Create Date: 2016-07-19 12:15:23.760153

"""

# revision identifiers, used by Alembic.
revision = '16627338400e'
down_revision = '2daf3bc83d75'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer))
)

card_groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('auto_assign_user', sa.String)
)

def upgrade():
    conn = op.get_bind()

    q = sa.select([card_groups.c.id, card_groups.c.auto_assign_user, card_groups.c.parent_id])\
            .where(card_groups.c.auto_assign_user!=None)

    card_q = sa.select([cards.c.id, cards.c.parent_id, cards.c.parents_ids])\
            .where(cards.c.id==sa.bindparam('cid'))

    attr_q = sa.text("""select a.title from attributes a
                    join attribute_groups ag on ag.id = a.group_id
                    join card_models cm on cm.attribute_group_id = ag.id
                    join card_enabled_models cem on cem.model_id = cm.id
                    join cards p on p.id = cem.card_id
                    join cards c on c.parents_ids[1] = p.id
                    where c.id = :cid and a.name = :name limit 1""")

    attr_top_q = sa.text("""select a.title from attributes a
                    join attribute_groups ag on ag.id = a.group_id
                    join card_models cm on cm.attribute_group_id = ag.id
                    join card_enabled_models cem on cem.model_id = cm.id
                    join cards p on p.id = cem.card_id
                    where p.id = :cid and a.name = :name limit 1""")

    updates = []
    for group in conn.execute(q):
        card = conn.execute(card_q, cid=group.parent_id).first()
        if not card:
            continue
        attr_title = conn.execute(attr_q if card.parent_id else attr_top_q,
            cid=card.id, name=group.auto_assign_user).scalar()
        updates.append({'gid': group.id, 'auto_assign_user': attr_title})

    if updates:
        conn.execute(card_groups.update().where(card_groups.c.id==sa.bindparam('gid')).values(
            auto_assign_user=sa.bindparam('auto_assign_user')), updates)

    op.execute("UPDATE card_attributes SET name = NULL WHERE attribute_id IS NOT NULL")
    op.execute("UPDATE attributes SET name = title")
    op.drop_column('attributes', 'title')
    op.create_index(op.f('ix_attributes_name'), 'attributes', ['name'], unique=False)

    op.add_column('views', sa.Column('name', sa.String(), nullable=True))
    op.execute("UPDATE views SET name = title")
    op.drop_column('views', 'title')

    op.create_table('card_log_types',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=False),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('attribute_group_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['attribute_group_id'], ['attribute_groups.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_log_types_card_id'), 'card_log_types', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_log_types_name'), 'card_log_types', ['name'], unique=False)

    op.create_table('card_logs',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=False),
        sa.Column('log_type_id', sa.Integer(), nullable=False),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['log_type_id'], ['card_log_types.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_logs_card_id'), 'card_logs', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_logs_log_type_id'), 'card_logs', ['log_type_id'], unique=False)

    op.create_table('card_log_values',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('log_id', sa.Integer(), nullable=False),
        sa.Column('type_name', sa.String(), nullable=True),
        sa.Column('attribute_id', sa.Integer(), nullable=True),
        sa.Column('value', postgresql.JSONB(), nullable=True),
        sa.Column('unindexed_value', sa.Text(), nullable=True),
        sa.ForeignKeyConstraint(['log_id'], ['card_logs.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['attribute_id'], ['attributes.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_log_values_log_id'), 'card_log_values', ['log_id'], unique=False)
    op.create_index(op.f('ix_card_log_values_attribute_id'), 'card_log_values', ['attribute_id'], unique=False)
    op.create_index(op.f('ix_card_log_values_value'), 'card_log_values', ['value'], unique=False)


def downgrade():
    pass



"""date_attribute_options_change

Revision ID: 0269
Revises: 0268
Create Date: 2025-03-25 11:49:05.967184

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0269'
down_revision = '0268'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE attributes
        SET options = (
            CASE
                WHEN jsonb_typeof(options) = 'object' OR options IS NULL THEN
                    (
                        jsonb_set(
                            jsonb_set(
                                COALESCE(options, '{}'::jsonb),
                                '{precision}',
                                CASE
                                    WHEN (COALESCE(options, '{}'::jsonb) ->> 'time')::boolean THEN
                                        '"days_time"'::jsonb
                                    ELSE
                                        '"days"'::jsonb
                                END,
                                true
                            ),
                            '{daterange}',
                            CASE
                                WHEN NOT (COALESCE(options, '{}'::jsonb) ? 'daterange') THEN
                                    'true'::jsonb
                                ELSE
                                    COALESCE(options, '{}'::jsonb)->'daterange'
                            END,
                            true
                        )
                    ) - 'time'
                ELSE
                    options
            END
        )
        WHERE type_name = 'date';
    """)


def downgrade():
    pass



"""add_monthly_tasks

Revision: 0112
Revision ID: edc4da1f645
Revises: 54c1c3cc20fd
Create Date: 2018-02-21 17:01:09.750527

"""

# revision identifiers, used by Alembic.
revision = 'edc4da1f645'
down_revision = '54c1c3cc20fd'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
from sqlalchemy.sql.expression import bindparam


def upgrade():
    conn = op.get_bind()
    op.execute("UPDATE recurring_tasks SET recurring_days = regexp_replace(recurring_days, '\d', 'weekly:\&', 'g') WHERE recurring_days != 'daily'")
    op.alter_column('recurring_tasks', 'recurring_days', new_column_name='recurrence', nullable=True)


def downgrade():
    conn = op.get_bind()
    op.alter_column('recurring_tasks', 'recurrence', new_column_name='recurring_days', nullable=True)
    op.execute("UPDATE recurring_tasks SET recurring_days = regexp_replace(recurring_days, 'weekly:(\d)', '\\1', 'g') WHERE recurring_days != 'daily'")



"""org_featured_url

Revision: 0220
Revision ID: 1a49f15e6ab4
Revises: 233b7875868c
Create Date: 2021-09-09 17:07:42.079878

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '1a49f15e6ab4'
down_revision = '233b7875868c'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organization_featured_items', sa.Column('url', sa.Text(), nullable=True))
    op.add_column('organization_featured_items', sa.Column('title', sa.Text(), nullable=True))
    op.add_column('organization_featured_items', sa.Column('favicon', sa.Text(), nullable=True))


def downgrade():
    op.drop_column('organization_featured_items', 'url')
    op.drop_column('organization_featured_items', 'title')
    op.drop_column('organization_featured_items', 'favicon')



"""group_type_builtin

Revision: 0031
Revision ID: 3db4ee6d1d77
Revises: 11c328aaee9c
Create Date: 2015-11-06 13:15:47.394461

"""

# revision identifiers, used by Alembic.
revision = '3db4ee6d1d77'
down_revision = '11c328aaee9c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_group_types', sa.Column('builtin', sa.String(), nullable=True))
    op.add_column('card_group_types', sa.Column('copied_from_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'card_group_types', 'card_group_types', ['copied_from_id'], ['id'], ondelete='SET NULL')
    op.create_index(op.f('ix_card_group_types_copied_from_id'), 'card_group_types', ['copied_from_id'], unique=False)

    op.execute("UPDATE card_group_types SET builtin = upper(name) WHERE name in ('Lists', 'Milestones')")

def downgrade():
    op.drop_index(op.f('ix_card_group_types_copied_from_id'), table_name='card_group_types')
    op.drop_column('card_group_types', 'copied_from_id')
    op.drop_column('card_group_types', 'builtin')



"""org_stat_counter

Revision: 0221
Revision ID: 27f0fec65f2e
Revises: 1a49f15e6ab4
Create Date: 2021-10-08 13:29:21.339263

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '27f0fec65f2e'
down_revision = '1a49f15e6ab4'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('organization_stat_counters',
        sa.Column('org_id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(), nullable=False),
        sa.Column('total', sa.Integer(), nullable=False),
        sa.Column('period', sa.String(), nullable=False),
        sa.Column('periods_count', postgresql.JSON(), nullable=True),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('org_id', 'name')
    )


def downgrade():
    op.drop_table('organization_stat_counters')



"""top_level_card_attribute_group

Revision ID: 0259
Revises: 0258
Create Date: 2023-04-04 12:30:25.886437

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import datetime


# revision identifiers, used by Alembic.
revision = '0259'
down_revision = '0258'
branch_labels = None
depends_on = None


organizations = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer()),
    sa.Column('top_level_card_attribute_group_id', sa.Integer())
)

organization_members = sa.Table(
    'organization_members',
    sa.MetaData(),
    sa.Column('org_id', sa.Integer()),
    sa.Column('user_id', sa.Integer()),
    sa.Column('permission', sa.Integer())
)

users = sa.Table(
    'users',
    sa.MetaData(),
    sa.Column('id', sa.Integer()),
    sa.Column('locale', sa.String())
)

attribute_groups = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer(), primary_key=True),
    sa.Column('owner_id', sa.Integer()),
    sa.Column('created_at', sa.DateTime()),
)

attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('group_id', sa.Integer()),
    sa.Column('name', sa.String()),
    sa.Column('type_name', sa.String()),
    sa.Column('container', sa.String()),
    sa.Column('position', sa.Integer())
)

builtin_attributes = {
    'en': [
        {'name': 'Description', 'type': 'content', 'container': '__top__', 'position': 0},
        {'name': 'Due Date', 'type': 'date', 'position': 0}
    ],
    'fr': [
        {'name': 'Description', 'type': 'content', 'container': '__top__', 'position': 0},
        {'name': "Date d'échéance", 'type': 'date', 'position': 0}
    ],
    'es': [
        {'name': 'Descripción', 'type': 'content', 'container': '__top__', 'position': 0},
        {'name': "Fecha de vencimiento", 'type': 'date', 'position': 0}
    ],
    'de': [
        {'name': 'Beschreibung', 'type': 'content', 'container': '__top__', 'position': 0},
        {'name': "Fälligkeitsdatum", 'type': 'date', 'position': 0}
    ],
    'pt': [
        {'name': 'Descrição', 'type': 'content', 'container': '__top__', 'position': 0},
        {'name': "Data de vencimento", 'type': 'date', 'position': 0}
    ],
    'it': [
        {'name': 'Descrizione', 'type': 'content', 'container': '__top__', 'position': 0},
        {'name': "Data di scadenza", 'type': 'date', 'position': 0}
    ]
}


def upgrade():
    op.add_column('organizations', sa.Column('top_level_card_attribute_group_id', sa.Integer()))
    op.create_foreign_key(None, 'organizations', 'attribute_groups', ['top_level_card_attribute_group_id'], ['id'], ondelete='SET NULL')

    conn = op.get_bind()
    for org in conn.execute(organizations.select().order_by(organizations.c.id)):
        locale = conn.execute(sa.select([users.c.locale]).where(sa.and_(
            organization_members.c.org_id==org.id, organization_members.c.user_id==users.c.id,
            organization_members.c.permission==4))).scalar()
        if not locale:
            locale = 'en'
        r = conn.execute(attribute_groups.insert().values(owner_id=org.id, created_at=datetime.datetime.utcnow()))
        group_id = r.inserted_primary_key[0]
        conn.execute(organizations.update().where(organizations.c.id==org.id).values(top_level_card_attribute_group_id=group_id))
        for attr in builtin_attributes.get(locale, builtin_attributes['en']):
            conn.execute(attributes.insert().values(group_id=group_id, name=attr['name'], type_name=attr['type'],
                                                    container=attr.get('container'), position=attr['position']))


def downgrade():
    op.drop_column('organizations', 'top_level_card_attribute_group_id')



"""org_files

Revision: 0063
Revision ID: 2a74262efd7d
Revises: 3ab6d84b6186
Create Date: 2016-06-24 11:05:23.286438

"""

# revision identifiers, used by Alembic.
revision = '2a74262efd7d'
down_revision = '3ab6d84b6186'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.rename_table('card_files', 'files')

    org_files = op.create_table('organizations_files',
        sa.Column('file_id', sa.Integer(), nullable=False),
        sa.Column('org_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['file_id'], ['files.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.UniqueConstraint('file_id', 'org_id')
    )

    op.execute("""INSERT INTO organizations_files (file_id, org_id) SELECT DISTINCT files.id, c.owner_id
        FROM files, card_attributes ca JOIN cards c ON c.id = ca.card_id
        WHERE json_build_array(files.id)::jsonb <@ ca.value AND ca.type_name = 'files'""")

    op.execute("UPDATE activities SET sub_object_type = 'File' WHERE sub_object_type = 'CardFile'")

    op.drop_table('cards_card_files')


def downgrade():
    pass



"""clear_card_attributes

Revision: 0170
Revision ID: 27a5b637092e
Revises: 21cd415ba55b
Create Date: 2019-07-04 16:34:26.117473

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '27a5b637092e'
down_revision = '21cd415ba55b'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""UPDATE cards SET cache_id = uuid_generate_v4() FROM card_attributes WHERE card_attributes.card_id = cards.id AND card_attributes.type_name = 'group_type'""")
    op.execute("""DELETE FROM card_attributes WHERE type_name = 'group_type'""")


def downgrade():
    pass



"""project_auto_actions

Revision: 0175
Revision ID: 4d90b902c3e0
Revises: ace95229377
Create Date: 2019-10-02 22:15:22.803369

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '4d90b902c3e0'
down_revision = 'ace95229377'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('project_auto_actions',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('group_id', sa.Integer(), nullable=True),
        sa.Column('on_events', postgresql.ARRAY(sa.String()), nullable=True),
        sa.Column('filter_condition', sa.String(), nullable=True),
        sa.Column('action_name', sa.String(), nullable=True),
        sa.Column('options', postgresql.JSONB(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['group_id'], ['card_groups.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_project_auto_actions_project_id'), 'project_auto_actions', ['project_id'], unique=False)
    op.create_index(op.f('ix_project_auto_actions_project_id_group_id'), 'project_auto_actions', ['project_id', 'group_id'], unique=False)

    # actions on groups from contexts
    op.execute("""
        INSERT INTO project_auto_actions (project_id, created_at, updated_at, created_by_id, group_id, action_name, options)
        SELECT p.id, cga.created_at, cga.updated_at, cga.created_by_id, cga.group_id, cga.action_name, cga.options
        FROM card_group_actions cga
        JOIN card_groups cg ON cg.id = cga.group_id
        JOIN card_group_types cgt ON cgt.id = cg.group_type_id
        JOIN project_enabled_group_types pegt ON pegt.group_type_id = cgt.id
        JOIN projects p ON p.id = pegt.project_id
    """)

    # actions on groups from project attributes
    op.execute("""
        INSERT INTO project_auto_actions (project_id, created_at, updated_at, created_by_id, group_id, action_name, options)
        SELECT p.id, cga.created_at, cga.updated_at, cga.created_by_id, cga.group_id, cga.action_name, cga.options
        FROM card_group_actions cga
        JOIN card_groups cg ON cg.id = cga.group_id
        JOIN card_group_types cgt ON cgt.id = cg.group_type_id
        JOIN attributes a ON a.id = cgt.attribute_id
        JOIN projects p ON p.attribute_group_id = a.group_id
    """)

    # actions on groups from model attributes
    op.execute("""
        INSERT INTO project_auto_actions (project_id, created_at, updated_at, created_by_id, group_id, action_name, options)
        SELECT p.id, cga.created_at, cga.updated_at, cga.created_by_id, cga.group_id, cga.action_name, cga.options
        FROM card_group_actions cga
        JOIN card_groups cg ON cg.id = cga.group_id
        JOIN card_group_types cgt ON cgt.id = cg.group_type_id
        JOIN attributes a ON a.id = cgt.attribute_id
        JOIN card_models cm ON cm.attribute_group_id = a.group_id
        JOIN project_enabled_models pem ON pem.model_id = cm.id
        JOIN projects p ON p.id = pem.project_id
    """)

    op.execute("UPDATE card_groups SET cache_id = generate_cache_id(id)")

    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'MANAGE_AUTO_ACTIONS')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin = 'ADMIN'
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"MANAGE_AUTO_ACTIONS"}'::varchar[])
    """)

    op.execute("""
        UPDATE roles SET cache_id = uuid_generate_v4()
        WHERE roles.id IN (
            SELECT DISTINCT roles.id
            FROM roles, role_permissions
            WHERE role_permissions.role_id = roles.id
            AND roles.builtin = 'ADMIN'
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"MANAGE_AUTO_ACTIONS"}'::varchar[])
        )
    """)


def downgrade():
    op.drop_table('project_auto_actions')



"""fix_models_attribute_groups

Revision: 0114
Revision ID: 4b8f6f8c1e02
Revises: 4d7f1ad39d84
Create Date: 2018-03-02 14:28:19.346050

"""

# revision identifiers, used by Alembic.
revision = '4b8f6f8c1e02'
down_revision = '4d7f1ad39d84'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import uuid
from datetime import datetime

enabled_models = sa.Table(
    'project_enabled_models',
    sa.MetaData(),
    sa.Column('project_id', sa.Integer),
    sa.Column('model_id', sa.Integer)
)

models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('cache_id', sa.String),
    sa.Column('copied_from_id', sa.Integer)
)

attribute_groups = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_at', sa.DateTime),
    sa.Column('shared_context', sa.PickleType),
    sa.Column('containers', postgresql.JSONB())
)

attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('description', sa.String),
    sa.Column('type_name', sa.String),
    sa.Column('container', sa.String),
    sa.Column('position', sa.Integer),
    sa.Column('options', sa.PickleType),
    sa.Column('default_value', sa.PickleType),
    sa.Column('is_from_builtin', sa.Boolean),
    sa.Column('show_on_tile', sa.Boolean),
    sa.Column('group_id', sa.Integer)
)

group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_at', sa.DateTime),
    sa.Column('updated_at', sa.DateTime),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('scope', sa.String),
    sa.Column('one_group_per_card', sa.Boolean),
    sa.Column('singular_name', sa.String),
    sa.Column('plural_name', sa.String),
    sa.Column('icon', sa.String),
    sa.Column('builtin', sa.String),
    sa.Column('display_on_card', sa.String),
    sa.Column('auto_random_color', sa.Boolean),
    sa.Column('cache_id', sa.String),
    sa.Column('auto_enabled_on_new_projects', sa.Boolean),
    sa.Column('is_project_specific', sa.Boolean),
    sa.Column('auto_assign_period', sa.String),
    sa.Column('attribute_id', sa.Integer)
)

groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('cache_id', sa.String)
)

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer)
)

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('attribute_id', sa.Integer)
)

def upgrade():
    update_models = []
    update_groups = []
    update_card_attributes = []

    conn = op.get_bind()

    # Attributes without attribute groups
    conn.execute(attributes.delete().where(attributes.c.group_id == None))

    # Models without attribute groups
    q_models = sa.text("""
        SELECT DISTINCT m.id, m.owner_id
        FROM card_models m, card_models m2
        WHERE m2.copied_from_id = m.id
        AND m.attribute_group_id IS NULL
        ORDER BY m.id
    """)
    for model in conn.execute(q_models):
        other_attribute_group_id = sa.select([models.c.attribute_group_id]).where(sa.and_(
            models.c.copied_from_id == model[0],
            models.c.attribute_group_id != None
        )).limit(1)
        attr_group = conn.execute(attribute_groups.select().where(attribute_groups.c.id == other_attribute_group_id)).first()
        if attr_group:
            res_attr_group = conn.execute(attribute_groups.insert().values(
                owner_id=model[1],
                created_at=datetime.utcnow(),
                shared_context=attr_group[3],
                containers=attr_group[4]))
            attr_group_id = res_attr_group.inserted_primary_key[0]
            update_models.append({'model_id': model[0], 'attr_group_id': attr_group_id, 'cache_id': uuid.uuid4()})
            for attr in conn.execute(attributes.select().where(attributes.c.group_id == attr_group[0])):
                if attr[12] is None:
                    continue
                res_attrs = conn.execute(attributes.insert().values(
                    card_model_id = attr[1],
                    card_id = attr[2],
                    name = attr[3],
                    description = attr[4],
                    type_name = attr[5],
                    container = attr[6],
                    position = attr[7],
                    options = attr[8],
                    default_value = attr[9],
                    is_from_builtin = attr[10],
                    show_on_tile = attr[11],
                    group_id = attr_group_id))
                attribute_id = res_attrs.inserted_primary_key[0]
                if attr[5] == 'group_type':
                    group_type = conn.execute(group_types.select().where(group_types.c.attribute_id == attr[0])).first()
                    res_gts = conn.execute(group_types.insert().values(
                        created_at = datetime.utcnow(),
                        created_by_id = group_type[3],
                        owner_id = model[1],
                        name = group_type[5],
                        description = group_type[6],
                        scope = group_type[7],
                        one_group_per_card = group_type[8],
                        singular_name = group_type[9],
                        plural_name = group_type[10],
                        icon = group_type[11],
                        builtin = group_type[12],
                        display_on_card = group_type[13],
                        auto_random_color = group_type[14],
                        cache_id = uuid.uuid4(),
                        auto_enabled_on_new_projects = group_type[16],
                        is_project_specific = group_type[17],
                        auto_assign_period = group_type[18],
                        attribute_id = attribute_id))

    # Models with duplicate attribute groups
    q_models = sa.text("""
        SELECT m.id, m.attribute_group_id, m.owner_id, pem.project_id
        FROM card_models m, project_enabled_models pem
        WHERE m.id = pem.model_id
        AND attribute_group_id IN (
                SELECT attribute_group_id from card_models
                WHERE attribute_group_id IS NOT NULL
                GROUP BY attribute_group_id
                HAVING count(*) > 1
        )
        AND copied_from_id IS NOT NULL
        ORDER BY attribute_group_id, id
    """)
    for model in conn.execute(q_models):
        attr_group = conn.execute(attribute_groups.select().where(attribute_groups.c.id == model[1])).first()
        if attr_group:
            project_id = model[3]
            res_attributes = conn.execute(attributes.select().where(attributes.c.group_id == model[1]))
            attr_group_id = None
            if res_attributes.rowcount > 0:
                res_attr_group = conn.execute(attribute_groups.insert().values(
                    owner_id=model[2],
                    created_at=datetime.utcnow(),
                    shared_context=attr_group[3],
                    containers=attr_group[4]))
                attr_group_id = res_attr_group.inserted_primary_key[0]
            update_models.append({'model_id': model[0], 'attr_group_id': attr_group_id, 'cache_id': uuid.uuid4()})
            if res_attributes.rowcount == 0:
                continue
            for attr in res_attributes:
                if attr[12] is None:
                    continue
                res_attrs = conn.execute(attributes.insert().values(
                    card_model_id = attr[1],
                    card_id = attr[2],
                    name = attr[3],
                    description = attr[4],
                    type_name = attr[5],
                    container = attr[6],
                    position = attr[7],
                    options = attr[8],
                    default_value = attr[9],
                    is_from_builtin = attr[10],
                    show_on_tile = attr[11],
                    group_id = attr_group_id))
                attribute_id = res_attrs.inserted_primary_key[0]
                if attr[5] == 'group_type':
                    group_type = conn.execute(group_types.select().where(group_types.c.attribute_id == attr[0])).first()
                    res_gt = conn.execute(group_types.insert().values(
                            created_at = datetime.utcnow(),
                            created_by_id = group_type[3],
                            owner_id = model[2],
                            name = group_type[5],
                            description = group_type[6],
                            scope = group_type[7],
                            one_group_per_card = group_type[8],
                            singular_name = group_type[9],
                            plural_name = group_type[10],
                            icon = group_type[11],
                            builtin = group_type[12],
                            display_on_card = group_type[13],
                            auto_random_color = group_type[14],
                            cache_id = uuid.uuid4(),
                            auto_enabled_on_new_projects = group_type[16],
                            is_project_specific = group_type[17],
                            auto_assign_period = group_type[18],
                            attribute_id = attribute_id))
                    gt_id = res_gt.inserted_primary_key[0]
                    q_groups = sa.text("""
                        SELECT g.id
                        FROM card_groups g, cards c, card_group_types gt, attributes a, attribute_groups ag, card_models m, project_enabled_models pem, projects p
                        WHERE g.parent_id = c.id
                        AND g.group_type_id = gt.id
                        AND gt.attribute_id = a.id
                        AND a.group_id = ag.id
                        AND ag.id = m.attribute_group_id
                        AND m.id = pem.model_id
                        AND c.project_id = p.id
                        AND m.is_project_specific is true
                        AND pem.project_id != c.project_id
                        AND gt.id = {0}
                        AND p.id = {1}
                    """.format(group_type[0], project_id))
                    for group in conn.execute(q_groups):
                        update_groups.append({'g_id': group[0], 'gt_id': gt_id, 'cache_id': uuid.uuid4()})
                else:
                    update_card_attributes.append({'old_a_id': attr[0], 'new_a_id': attribute_id, 'p_id': project_id})

    if update_models:
        conn.execute(models.update().where(models.c.id==sa.bindparam('model_id')).values(
            attribute_group_id=sa.bindparam('attr_group_id'), cache_id=sa.bindparam('cache_id')), update_models)

    if update_groups:
        conn.execute(groups.update().where(groups.c.id==sa.bindparam('g_id')).values(
            group_type_id=sa.bindparam('gt_id'), cache_id=sa.bindparam('cache_id')), update_groups)

    if update_card_attributes:
        conn.execute(card_attributes.update().where(sa.and_(card_attributes.c.card_id==cards.c.id, cards.c.project_id==sa.bindparam('p_id'), card_attributes.c.attribute_id==sa.bindparam('old_a_id'))).values(
            attribute_id=sa.bindparam('new_a_id')), update_card_attributes)


def downgrade():
    pass



"""user_display_name

Revision: 0060
Revision ID: 7a809a8d04e
Revises: 4cd3011ada05
Create Date: 2016-06-26 22:31:17.157882

"""

# revision identifiers, used by Alembic.
revision = '7a809a8d04e'
down_revision = '4cd3011ada05'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('display_name', sa.String(), nullable=True))


def downgrade():
    op.drop_column('users', 'display_name')



"""fix_cards_path

Revision: 0224
Revision ID: 58cf93323b46
Revises: 22b7db58107b
Create Date: 2021-10-29 16:43:51.119013

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import uuid


# revision identifiers, used by Alembic.
revision = '58cf93323b46'
down_revision = '22b7db58107b'
branch_labels = None
depends_on = None

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer)),
    sa.Column('cache_id', sa.String),
)

parents = {}

def find_parents(conn, card):
    if card.id in parents:
        return parents[card.id]

    parents_ids = []
    if len(card.parents_ids) == 1:
        parents_ids = [card.parent_id]
    else:
        cards2 = cards.alias()
        parent = conn.execute(cards2.select().where(sa.and_(cards2.c.id==cards.c.parent_id, cards.c.parent_id==card.parent_id))).first()
        parents_ids = list(find_parents(conn, parent))
        parents_ids.append(parent.id)

    parents[card.id] = parents_ids
    return parents_ids


def upgrade():
    conn = op.get_bind()

    for card in conn.execute(cards.select().where(sa.and_(cards.c.parent_id != None, sa.any_(cards.c.parents_ids) == -1) == None)):
        find_parents(conn, card)

    if parents:
        infos = []
        for cid, parents_ids in parents.items():
            infos.append({'cid': cid, 'parents_ids': parents_ids, 'cache_id':uuid.uuid4()})
        conn.execute(cards.update().where(cards.c.id==sa.bindparam('cid')).values(
            parents_ids=sa.bindparam('parents_ids'), cache_id=sa.bindparam('cache_id')), infos)


def downgrade():
    pass



"""count_working_days

Revision: 0197
Revision ID: 3b5a33bad9e7
Revises: e1180f3ae5fe
Create Date: 2020-11-12 15:41:33.637297

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3b5a33bad9e7'
down_revision = 'e1180f3ae5fe'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION count_working_days(from_date date, to_date date) RETURNS bigint
        AS $body$
            SELECT count(*)
            FROM generate_series(from_date, to_date, '1 day'::interval) d
            WHERE EXTRACT('isodow' from d) < 6
        $body$ LANGUAGE sql;
    """)


def downgrade():
    pass



"""view_group_by_lane

Revision: 0107
Revision ID: 3b7c458d8852
Revises: 187c2a740494
Create Date: 2017-10-14 12:36:53.838093

"""

# revision identifiers, used by Alembic.
revision = '3b7c458d8852'
down_revision = '187c2a740494'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('views', sa.Column('lane_group_type_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'views', 'card_group_types', ['lane_group_type_id'], ['id'], ondelete='SET NULL')

    op.add_column('views', sa.Column('lane_group_by_attribute_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'views', 'attributes', ['lane_group_by_attribute_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('views', 'lane_group_type_id')
    op.drop_column('views', 'lane_group_by_attribute_id')



"""option_ref

Revision: 0193
Revision ID: 60fef5bde116
Revises: b01140bf6710
Create Date: 2020-09-15 14:34:10.190593

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '60fef5bde116'
down_revision = 'b01140bf6710'
branch_labels = None
depends_on = None


views = sa.Table(
    'views',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('mode', sa.String),
    sa.Column('mode_options', postgresql.JSONB)
)


def upgrade():
    op.execute("""
        UPDATE project_auto_actions
        SET options = jsonb_build_object('group',
            jsonb_build_object('name', options->'group',
                'group_type', jsonb_build_object('name', options->'group_type'))
        )
        WHERE action_name = 'append_to_group'
    """)

    op.execute("""
        UPDATE project_auto_actions
        SET options = jsonb_build_object('request_form', options->'form_id')
        WHERE action_name = 'create_request_on_card'
    """)

    op.execute("""
        UPDATE project_auto_actions
        SET options = jsonb_build_object(
            'email_template', options->'template_id',
            'recipient_attr', options->'recipient_attr_id'
        )
        WHERE action_name = 'send_email'
    """)

    conn = op.get_bind()
    updates = []
    for view in conn.execute(views.select().where(views.c.mode == 'report')):
        if isinstance(view['mode_options'], dict) and view['mode_options'].get('widgets'):
            for widget in view['mode_options']['widgets']:
                if widget['chart'] == 'burndown_chart':
                    _rename_key(widget, 'group_type_id', 'group_type')
                    _rename_key(widget, 'group_id', 'group')
                    _rename_key(widget, 'points_attr_id', 'points_attr')
                elif widget['chart'] in ('groups', 'cards_per_group_breakdown', 'cycle_times', 'cumulative_flow_diagram'):
                    _rename_key(widget, 'group_type_id', 'group_type')
                elif widget['chart'] in ('time_between_two_groups', 'evolution_of_time_between_two_groups'):
                    _rename_key(widget, 'from_group_id', 'from_group')
                    _rename_key(widget, 'to_group_id', 'to_group')
                elif widget['chart'] == 'evolution_nb_cards_in_group':
                    _rename_key(widget, 'group_id', 'group')
                elif widget['chart'] in ('logs_count', 'logs_number_daily', 'logs_number_sum', 'logs_number_sum_by_user'):
                    _rename_key(widget, 'log_type_id', 'log_type')
                elif widget['chart'] == 'weekly_velocity_chart':
                    _rename_key(widget, 'points_attr_id', 'points_attr')
                elif widget['chart'] == 'groups_velocity_chart':
                    _rename_key(widget, 'group_type_id', 'group_type')
                    _rename_key(widget, 'points_attr_id', 'points_attr')
                elif widget['chart'] == 'members_velocity_chart':
                    _rename_key(widget, 'members_attr_id', 'members_attr')
                    _rename_key(widget, 'group_type_id', 'group_type')
                    _rename_key(widget, 'points_attr_id', 'points_attr')
                elif widget['chart'] == 'workload':
                    _rename_key(widget, 'ponderate_attr_id', 'ponderate_attr')
            updates.append({'vid': view['id'], 'opts': view['mode_options']})

    if updates:
        stmt = views.update().where(views.c.id==sa.bindparam('vid')).values(
            mode_options=sa.bindparam('opts'))
        conn.execute(stmt, updates)

def downgrade():
    pass


def _rename_key(widget, old_key, new_key):
    v = widget['options'].pop(old_key, None)
    if v:
        widget['options'][new_key] = v



"""updated_at_prop_for_views

Revision: 0122
Revision ID: 9e0835454e9
Revises: 3cb0832a6d79
Create Date: 2018-01-15 18:45:57.728177

"""

# revision identifiers, used by Alembic.
revision = '9e0835454e9'
down_revision = '3cb0832a6d79'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('views', sa.Column('updated_at', sa.DateTime(), nullable=True))

    op.execute("""UPDATE views SET updated_at = created_at""")


def downgrade():
    op.drop_column('views', 'updated_at')



"""clean_projects_models

Revision: 0205
Revision ID: 90dacf065be7
Revises: e2464817d291
Create Date: 2021-05-24 16:22:47.841626

"""
from alembic import op
import sqlalchemy as sa
import uuid
from sqlalchemy.dialects import postgresql
from datetime import datetime


# revision identifiers, used by Alembic.
revision = '90dacf065be7'
down_revision = 'e2464817d291'
branch_labels = None
depends_on = None

projects = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_by_id', sa.Integer)
)

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('default_child_card_model_id', sa.Integer),
    sa.Column('cache_id', sa.String),
    sa.Column('model_id', sa.Integer)
)

card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('name', sa.String),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_in_id', sa.Integer),
    sa.Column('is_project_specific', sa.String),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('copied_from_id', sa.Integer),
    sa.Column('indicator', sa.String),
    sa.Column('card_color', sa.String),
    sa.Column('cache_id', sa.String)
)

enabled_models = sa.Table(
    'project_enabled_models',
    sa.MetaData(),
    sa.Column('project_id', sa.Integer),
    sa.Column('model_id', sa.Integer)
)

attribute_groups = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_at', sa.DateTime),
    sa.Column('shared_context', sa.PickleType),
    sa.Column('containers', postgresql.JSONB())
)

attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('description', sa.String),
    sa.Column('type_name', sa.String),
    sa.Column('container', sa.String),
    sa.Column('position', sa.Integer),
    sa.Column('options', sa.PickleType),
    sa.Column('default_value', sa.PickleType),
    sa.Column('is_from_builtin', sa.Boolean),
    sa.Column('show_on_tile', sa.Boolean),
    sa.Column('group_id', sa.Integer),
    sa.Column('is_pii', sa.Boolean)
)

group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_at', sa.DateTime),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('scope', sa.String),
    sa.Column('one_group_per_card', sa.Boolean),
    sa.Column('singular_name', sa.String),
    sa.Column('plural_name', sa.String),
    sa.Column('icon', sa.String),
    sa.Column('builtin', sa.String),
    sa.Column('display_on_card', sa.String),
    sa.Column('auto_random_color', sa.Boolean),
    sa.Column('cache_id', sa.String),
    sa.Column('auto_enabled_on_new_projects', sa.Boolean),
    sa.Column('is_project_specific', sa.Boolean),
    sa.Column('auto_assign_period', sa.String),
    sa.Column('attribute_id', sa.Integer)
)

groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_at', sa.DateTime),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('position', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('description', sa.String),
    sa.Column('is_archived', sa.Boolean),
    sa.Column('archived_at', sa.DateTime),
    sa.Column('meta', sa.PickleType),
    sa.Column('color', sa.String),
    sa.Column('icon', sa.String),
    sa.Column('show_flow_indicator', sa.Boolean),
    sa.Column('card_state', sa.String),
    sa.Column('default_card_model_id', sa.Integer),
    sa.Column('max_cards_limit', sa.Integer),
    sa.Column('disallow_direct_card_creation', sa.Boolean),
    sa.Column('start_date', sa.Date),
    sa.Column('end_date', sa.Date),
    sa.Column('always_show_description', sa.Boolean),
    sa.Column('parent_id', sa.Integer),
    sa.Column('cache_id', sa.String)
)

cgc = sa.Table(
    'card_groups_cards',
    sa.MetaData(),
    sa.Column('card_id', sa.Integer),
    sa.Column('group_id', sa.Integer),
    sa.Column('position', sa.Integer)
)

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('attribute_id', sa.Integer)
)

activities = sa.Table(
    'activities',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer),
    sa.Column('object_type', sa.String),
    sa.Column('object_id', sa.Integer),
    sa.Column('target_type', sa.String),
    sa.Column('target_id', sa.Integer)
)

def get_from_list(elements, props, values):
    ret = None
    for _el in elements:
        for i in range(len(props)):
            ret = _el
            if _el[props[i]] != values[i]:
                ret = None
                break
        if ret:
            return ret
    return None

def upgrade():
    update_cards = []
    update_cards2 = []
    update_gts = []
    update_card_attributes = []
    update_cgc = []
    delete_cgc = []
    update_activities = []

    conn = op.get_bind()

    # look for projects where there is model ids in default_child_card_model_id that are not enabled in the project
    # this is the result of a bug during copy/move operations
    models_used_as_default_child_card_model_but_not_enabled_query = sa.select([
        card_models.c.id,
        card_models.c.name,
        card_models.c.owner_id,
        card_models.c.is_project_specific,
        card_models.c.attribute_group_id,
        card_models.c.indicator,
        card_models.c.card_color,
        projects.c.id,
        projects.c.owner_id,
        projects.c.created_by_id,
        cards.c.id
    ]).select_from(
        card_models\
            .join(cards, cards.c.default_child_card_model_id==card_models.c.id)\
            .join(projects, projects.c.id==cards.c.project_id)
    ).where(
        card_models.c.id.notin_(
            sa.select([enabled_models.c.model_id]).where(enabled_models.c.project_id==projects.c.id)
        )
    )

    for row in conn.execute(models_used_as_default_child_card_model_but_not_enabled_query):
        model_id = row[0]
        model_name = row[1]
        model_owner_id = row[2]
        model_is_project_specific = row[3]
        model_attribute_group_id = row[4]
        model_indicator = row[5]
        model_color = row[6]
        project_id = row[7]
        project_owner_id = row[8]
        project_created_by_id = row[9]
        card_id = row[10]

        if not model_is_project_specific and model_owner_id == project_owner_id:
            conn.execute(enabled_models.insert().values(project_id=project_id, model_id=model_id))
            conn.execute(card_models.update().where(card_models.c.id==model_id).values(cache_id=uuid.uuid4()))

        else:
            other_model_id = None
            other_model_attribute_group_id = None
            other_model_attrs = []

            # we assume that a model with the same name in the project can be used as replacement
            same_name_replacement_model = conn.execute(sa.select([
                card_models.c.id,
                card_models.c.attribute_group_id
            ]).select_from(
                card_models.join(enabled_models, enabled_models.c.model_id==card_models.c.id)
            ).where(
                sa.and_(card_models.c.name==model_name, enabled_models.c.project_id==project_id)
            )).first()

            if same_name_replacement_model:
                # replace model with one with same name in project and rewire attributes on cards
                other_model_id = same_name_replacement_model[0]
                other_model_attribute_group_id = same_name_replacement_model[1]
                for attr in conn.execute(attributes.select().where(attributes.c.group_id == other_model_attribute_group_id)):
                    other_model_attrs.append({"id": attr[0], "name": attr[3], "type_name": attr[5]})

            else:
                # no replacement model found, we create a new project specific model as a copy of the original model
                attr_group = conn.execute(attribute_groups.select().where(attribute_groups.c.id == model_attribute_group_id)).first()
                if attr_group:
                    attr_group_infos = conn.execute(attribute_groups.insert().values(
                        owner_id=project_owner_id,
                        created_at=datetime.utcnow(),
                        shared_context=attr_group[3],
                        containers=attr_group[4]))
                    other_model_attribute_group_id = attr_group_infos.inserted_primary_key[0]
                    other_model_infos = conn.execute(card_models.insert().values( name=model_name,
                                                                            owner_id=project_owner_id,
                                                                            created_in_id=project_id,
                                                                            is_project_specific=True,
                                                                            attribute_group_id=other_model_attribute_group_id,
                                                                            copied_from_id=model_id,
                                                                            indicator=model_indicator,
                                                                            card_color=model_color,
                                                                            cache_id = uuid.uuid4()))
                    other_model_id = other_model_infos.inserted_primary_key[0]
                    conn.execute(enabled_models.insert().values(project_id=project_id, model_id=other_model_id))

            # other_model_id can remain None after the above steps if the original model was deleted

            update_cards.append({'card_id': card_id, 'default_child_card_model_id': other_model_id, 'cache_id': uuid.uuid4()})

            # rewire or re-create attributes in cards after model change above
            for attr in conn.execute(attributes.select().where(attributes.c.group_id == model_attribute_group_id)):
                other_model_attr = get_from_list(other_model_attrs, ["name", "type_name"], [attr[3], attr[5]])
                other_attribute_id = None
                if other_model_attr:
                    other_attribute_id = other_model_attr["id"]
                    update_card_attributes.append({'old_attr_id': attr[0], 'new_attr_id': other_model_attr["id"], '_project_id': project_id})
                else:
                    attr_infos = conn.execute(attributes.insert().values(
                        card_model_id = attr[1],
                        card_id = attr[2],
                        name = attr[3],
                        description = attr[4],
                        type_name = attr[5],
                        container = attr[6],
                        position = attr[7],
                        options = attr[8],
                        default_value = attr[9],
                        is_from_builtin = attr[10],
                        show_on_tile = attr[11],
                        group_id = other_model_attribute_group_id,
                        is_pii = attr[13]))
                    other_attribute_id = attr_infos.inserted_primary_key[0]
                    other_model_attrs.append({"id": other_attribute_id, "name": attr[3], "type_name": attr[5]})

                if attr[5] == 'group_type':
                    group_type = conn.execute(group_types.select().where(group_types.c.attribute_id == attr[0])).first()
                    other_group_type_id = None
                    other_groups = []
                    if other_model_attr:
                        other_group_type_id = conn.execute(group_types.select().where(group_types.c.attribute_id == other_model_attr["id"])).first()[0]
                        update_gts.append({'gt_id': other_group_type_id, 'attr_id': other_attribute_id, 'cache_id': uuid.uuid4()})
                        for group_info in conn.execute(groups.select().where(groups.c.group_type_id == other_group_type_id)):
                            other_groups.append({'id': group_info[0], 'title': group_info[5]})
                    else:
                        gt_infos = conn.execute(group_types.insert().values(
                            created_at = datetime.utcnow(),
                            created_by_id = project_created_by_id,
                            owner_id = project_owner_id,
                            name = group_type[4],
                            description = group_type[5],
                            scope = group_type[6],
                            one_group_per_card = group_type[7],
                            singular_name = group_type[8],
                            plural_name = group_type[9],
                            icon = group_type[10],
                            builtin = group_type[11],
                            display_on_card = group_type[12],
                            auto_random_color = group_type[13],
                            cache_id = uuid.uuid4(),
                            auto_enabled_on_new_projects = group_type[15],
                            is_project_specific = group_type[16],
                            auto_assign_period = group_type[17],
                            attribute_id = other_attribute_id))
                        other_group_type_id = gt_infos.inserted_primary_key[0]

                    for group in conn.execute(groups.select().where(groups.c.group_type_id == group_type[0])):
                        other_group = get_from_list(other_groups, ["title"], [group[5]])
                        if other_group:
                            other_group_id = other_group['id']
                            for child in conn.execute(cards.select().where(cards.c.parent_id == card_id)):
                                if conn.execute(cgc.select().where(sa.and_(cgc.c.group_id == other_group_id, cgc.c.card_id == child[0]))).first() is None:
                                    update_cgc.append({'_project_id': project_id, '_card_id': child[0], 'old_group_id': group[0], 'new_group_id': other_group_id})
                                else:
                                    delete_cgc.append({'_card_id': child[0], 'old_group_id': group[0]})
                        else:
                            g_infos = conn.execute(groups.insert().values(
                                created_at = datetime.utcnow(),
                                created_by_id = project_created_by_id,
                                group_type_id = other_group_type_id,
                                position = group[4],
                                title = group[5],
                                description = group[6],
                                is_archived = group[7],
                                archived_at = group[8],
                                meta = group[9],
                                color = group[10],
                                icon = group[11],
                                show_flow_indicator = group[12],
                                card_state = group[13],
                                default_card_model_id = group[14],
                                max_cards_limit = group[15],
                                disallow_direct_card_creation = group[16],
                                start_date = group[17],
                                end_date = group[18],
                                always_show_description = group[19],
                                parent_id = card_id,
                                cache_id = uuid.uuid4()
                            ))
                            other_group_id = g_infos.inserted_primary_key[0]
                            for child in conn.execute(cards.select().where(cards.c.parent_id == card_id)):
                                update_cgc.append({'_project_id': project_id, '_card_id': child[0], 'old_group_id': group[0], 'new_group_id': other_group_id})

                update_card_attributes.append({'old_attr_id': attr[0], 'new_attr_id': other_attribute_id, '_project_id': project_id})

            # note that if a replacement model could not be created, cards will lose their model
            update_cards2.append({'_project_id': project_id, 'old_model_id': model_id, 'new_model_id': other_model_id})

    if update_cgc:
        conn.execute(cgc.update().where(
            sa.and_(cgc.c.card_id==sa.bindparam('_card_id'), cgc.c.group_id==sa.bindparam('old_group_id'))
        ).values(group_id=sa.bindparam('new_group_id')), update_cgc)
        conn.execute(activities.update().where(
            sa.and_(activities.c.project_id==sa.bindparam('_project_id'),
                    activities.c.object_type=='Card',
                    activities.c.target_type=='CardGroup',
                    activities.c.target_id==sa.bindparam('old_group_id'))
        ).values(target_id=sa.bindparam('new_group_id')), update_cgc)

    if delete_cgc:
        stmt = cgc.delete().where(sa.and_(cgc.c.group_id == sa.bindparam('old_group_id'), cgc.c.card_id==sa.bindparam('_card_id')))
        conn.execute(stmt, delete_cgc)

    if update_gts:
        conn.execute(group_types.update().where(group_types.c.id==sa.bindparam('gt_id')).values(attribute_id=sa.bindparam('attr_id'), cache_id=sa.bindparam('cache_id')), update_gts)

    if update_cards:
        conn.execute(cards.update().where(cards.c.id==sa.bindparam('card_id')).values(default_child_card_model_id=sa.bindparam('default_child_card_model_id'), cache_id=sa.bindparam('cache_id')), update_cards)

    if update_cards2:
        conn.execute(cards.update().where(
            sa.and_(cards.c.project_id==sa.bindparam('_project_id'), cards.c.model_id==sa.bindparam('old_model_id'))
        ).values(model_id=sa.bindparam('new_model_id'), cache_id=sa.func.uuid_generate_v4()), update_cards2)

    if update_card_attributes:
        conn.execute(card_attributes.update().where(
            sa.and_(card_attributes.c.card_id==cards.c.id, cards.c.project_id==sa.bindparam('_project_id'), card_attributes.c.attribute_id==sa.bindparam('old_attr_id'))
        ).values(attribute_id=sa.bindparam('new_attr_id')), update_card_attributes)
        conn.execute(activities.update().where(
            sa.and_(activities.c.project_id==sa.bindparam('_project_id'), activities.c.object_type=='Attribute', activities.c.object_id==sa.bindparam('old_attr_id'))
        ).values(object_id=sa.bindparam('new_attr_id')), update_card_attributes)

    conn.execute("""
        WITH positions as (
            SELECT attributes.id, row_number() OVER (partition by attributes.group_id, attributes.container order by position) AS p
            FROM attributes JOIN card_models ON card_models.attribute_group_id = attributes.group_id
        )
        UPDATE attributes SET position = positions.p - 1 FROM positions
        WHERE positions.id = attributes.id
    """)
    conn.execute("""UPDATE card_models SET cache_id = uuid_generate_v4()""")


def downgrade():
    pass



"""org_invite_token

Revision: 0228
Revision ID: ad80214e4b12
Revises: cec3b3150599
Create Date: 2021-12-07 10:16:04.190989

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'ad80214e4b12'
down_revision = 'cec3b3150599'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('invite_token', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('invite_token_default_perm', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_organizations_invite_token'), 'organizations', ['invite_token'], unique=False)
    op.execute("UPDATE organizations SET invite_token = uuid_generate_v4(), invite_token_default_perm = 2")


def downgrade():
    op.drop_column('organizations', 'invite_token')
    op.drop_column('organizations', 'invite_token_default_perm')



"""integration_system_app

Revision: 0236
Revision ID: 5f92d1bc0cfa
Revises: 8abfd8e00867
Create Date: 2022-02-15 15:38:32.262715

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '5f92d1bc0cfa'
down_revision = '8abfd8e00867'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("UPDATE projects SET system_apps = array_append(system_apps, 'integrations')")


def downgrade():
    pass



"""card_position

Revision: 0181
Revision ID: 27a56466f9a7
Revises: 175395377f47
Create Date: 2020-01-06 11:26:47.187268

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '27a56466f9a7'
down_revision = '175395377f47'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('cards', sa.Column('position', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_cards_position'), 'cards', ['position'], unique=False)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by project_id order by id) AS p
            FROM cards
        ) UPDATE cards SET position = positions.p - 1 FROM positions
        WHERE positions.id = cards.id
    """)


def downgrade():
    op.drop_column('cards', 'position')



"""add_sharing_via_url

Revision: 0148
Revision ID: 1d8095d7022a
Revises: 44e27652773
Create Date: 2018-09-24 15:24:05.813512

"""

# revision identifiers, used by Alembic.
revision = '1d8095d7022a'
down_revision = '44e27652773'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    conn = op.get_bind()

    reminders = op.create_table('share_objects',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('token', sa.String(), nullable=False, unique=True),
        sa.Column('project_id', sa.Integer(), nullable=False),
        sa.Column('object_type', sa.String(), nullable=False),
        sa.Column('object_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_share_objects_token'), 'share_objects', ['token'], unique=True)
    op.create_index(op.f('ix_share_objects_object_type_object_id'), 'share_objects', ['object_type', 'object_id'], unique=False)

def downgrade():
    op.drop_table('share_objects')



"""added_cards_mongo_id_index

Revision: 0010
Revision ID: 5a7dc5f0f913
Revises: 3a5f18e53496
Create Date: 2015-06-22 09:18:44.239036

"""

# revision identifiers, used by Alembic.
revision = '5a7dc5f0f913'
down_revision = '3a5f18e53496'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_index(op.f('ix_cards_mongo_id'), 'cards', ['mongo_id'], unique=False)
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_cards_mongo_id'), table_name='cards')
    ### end Alembic commands ###



"""request_form_no_request

Revision: 0215
Revision ID: 3c258d81bd0c
Revises: 2e07de14ad8c
Create Date: 2021-06-03 17:47:50.252203

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3c258d81bd0c'
down_revision = '2e07de14ad8c'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('request_forms', sa.Column('request_page', sa.Boolean(), nullable=True))
    op.add_column('request_forms', sa.Column('email_required', sa.Boolean(), nullable=True))
    op.execute("UPDATE request_forms SET request_page = true, email_required = true")


def downgrade():
    op.drop_column('request_forms', 'request_page')
    op.drop_column('request_forms', 'email_required')



"""teams

Revision: 0186
Revision ID: 416d8e4d464b
Revises: d3e47057f56
Create Date: 2020-04-30 22:09:38.117394

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '416d8e4d464b'
down_revision = 'd3e47057f56'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('teams',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('org_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('slug', sa.String(), nullable=True),
        sa.Column('org_members_can_observe', sa.Boolean(), nullable=True),
        sa.Column('org_members_can_autojoin', sa.Boolean(), nullable=True),
        sa.Column('org_members_autojoin_perm', sa.Integer(), nullable=True),
        sa.Column('max_allowed_users', sa.Integer(), nullable=True),
        sa.Column('color', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_teams_org_id'), 'teams', ['org_id'], unique=False)
    op.create_index(op.f('ix_teams_slug'), 'teams', ['slug'], unique=True)

    op.create_table('team_members',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('team_id', sa.Integer(), nullable=True),
        sa.Column('org_member_id', sa.Integer(), nullable=False),
        sa.Column('permission', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['team_id'], ['teams.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['org_member_id'], ['organization_members.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_team_members_team_id'), 'team_members', ['team_id'], unique=False)
    op.create_index(op.f('ix_team_members_team_id_permission'), 'team_members', ['team_id', 'permission'], unique=False)
    op.create_index(op.f('ix_team_members_team_id_org_member_id'), 'team_members', ['team_id', 'org_member_id'], unique=True)

    op.add_column('projects', sa.Column('team_id', sa.Integer(), nullable=True))
    op.create_foreign_key('projects_team_id_fkey', 'projects', 'teams', ['team_id'], ['id'], ondelete='SET NULL')
    op.create_index(op.f('ix_projects_team_id'), 'projects', ['team_id'], unique=False)
    op.add_column('projects', sa.Column('team_members_can_observe', sa.Boolean(), nullable=True))
    op.add_column('projects', sa.Column('team_members_can_autojoin', sa.Boolean(), nullable=True))
    op.execute("UPDATE projects SET team_members_can_observe = org_members_can_observe, team_members_can_autojoin = org_members_can_autojoin")

    op.add_column('organizations', sa.Column('logo_id', sa.Integer(), nullable=True))
    op.add_column('organizations', sa.Column('image_cover_id', sa.Integer(), nullable=True))
    op.add_column('organizations', sa.Column('cover_color', sa.String(), nullable=True))
    op.create_foreign_key('organizations_logo_id_fkey', 'organizations', 'files', ['logo_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key('organizations_image_cover_id_fkey', 'organizations', 'files', ['image_cover_id'], ['id'], ondelete='CASCADE')


def downgrade():
    op.drop_column('projects', 'team_id')
    op.drop_column('projects', 'team_members_can_observe')
    op.drop_column('projects', 'team_members_can_autojoin')
    op.drop_table('team_members')
    op.drop_table('teams')



"""date_attr_update_on_json_null_options

Revision ID: 0274
Revises: 0273
Create Date: 2025-07-11 09:36:08.483552

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0274'
down_revision = '0273'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE attributes
        SET options =
            jsonb_set(
                jsonb_set(
                    '{}'::jsonb,
                    '{precision}',
                    '"days"'::jsonb,
                    true
                ),
                '{daterange}',
                'true'::jsonb,
                true
            )
        WHERE type_name = 'date'
          AND options = 'null'::jsonb;
    """)


def downgrade():
    pass



"""signup_utmz

Revision: 0116
Revision ID: 525735b60e4d
Revises: 28ef45c94dfd
Create Date: 2018-03-13 15:00:57.887617

"""

# revision identifiers, used by Alembic.
revision = '525735b60e4d'
down_revision = '28ef45c94dfd'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.add_column('users', sa.Column('signup_utma', postgresql.JSONB(), nullable=True))
    op.add_column('users', sa.Column('signup_utmz', postgresql.JSONB(), nullable=True))
    op.create_index(op.f('ix_users_signup_utmz'), 'users', ['signup_utmz'], unique=False)


def downgrade():
    op.drop_column('users', 'signup_utmz')
    op.drop_column('users', 'signup_utma')



"""only_show_own_model_attributes

Revision ID: 0256
Revises: 0255
Create Date: 2024-03-29 20:35:37.091447

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0256'
down_revision = '0255'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('card_models', sa.Column('only_show_own_attributes', sa.Boolean()))
    op.execute("UPDATE card_models SET only_show_own_attributes = false")


def downgrade():
    op.drop_column('card_models', 'only_show_own_attributes')



"""share_object_forms

Revision: 0155
Revision ID: 284aac8ecbc6
Revises: 64703a6228b
Create Date: 2018-12-06 16:23:18.371420

"""

# revision identifiers, used by Alembic.
revision = '284aac8ecbc6'
down_revision = '64703a6228b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('share_object_forms',
        sa.Column('share_object_id', sa.Integer(), nullable=True),
        sa.Column('form_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['form_id'], ['request_forms.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['share_object_id'], ['share_objects.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_share_object_forms_form_id_share_object_id'), 'share_object_forms', ['form_id', 'share_object_id'], unique=False)
    op.add_column('share_objects', sa.Column('cache_id', sa.String(), nullable=True))


def downgrade():
    #op.drop_column('share_objects', 'cache_id')
    op.drop_index(op.f('ix_share_object_forms_form_id_share_object_id'), table_name='share_object_forms')
    op.drop_table('share_object_forms')



"""uuid

Revision: 0048
Revision ID: 3e7436bcd512
Revises: 3457fda0cf2a
Create Date: 2016-02-25 17:15:01.125712

"""

# revision identifiers, used by Alembic.
revision = '3e7436bcd512'
down_revision = '3457fda0cf2a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""CREATE FUNCTION generate_cache_id(id text) RETURNS text
        AS 'SELECT id || current_timestamp::text;' LANGUAGE SQL;""")
    op.execute("""CREATE FUNCTION generate_cache_id(id int) RETURNS text
        AS 'SELECT generate_cache_id(id::text);' LANGUAGE SQL;""")


def downgrade():
    op.execute("""DROP FUNCTION generate_cache_id(id int)""")
    op.execute("""DROP FUNCTION generate_cache_id(id text)""")



"""deduplicate_request_tokens

Revision: 0203
Revision ID: 397b2cb08fff
Revises: 47e876a62c34
Create Date: 2021-02-10 10:25:20.727266

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import csv
import uuid


# revision identifiers, used by Alembic.
revision = '397b2cb08fff'
down_revision = '47e876a62c34'
branch_labels = None
depends_on = None


requests = sa.Table(
    'requests',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('token', sa.String)
)


def upgrade():
    conn = op.get_bind()
    q = sa.text("""select distinct r1.id, r1.token, coalesce(r1.email, u.email) as email, f.name as form_name, f.project_id
        from requests r1 join request_forms f on f.id = r1.request_form_id left join users u on u.id = r1.user_id, requests r2
        where r1.id > r2.id and r1.token = r2.token order by id""")

    updates = []
    loglines = []
    for r in conn.execute(q):
        token = uuid.uuid4()
        loglines.append([r['id'], r['token'], token, r['email'], r['form_name'], r['project_id']])
        updates.append({'rid': r['id'], 'token': token})

    if loglines:
        with open('requests-duplicates.csv', 'w') as logfile:
            w = csv.writer(logfile)
            w.writerow(["request_id", "old_token", "new_token", "email", "form_name", "project_id"])
            w.writerows(loglines)

    if updates:
        stmt = requests.update().where(requests.c.id==sa.bindparam('rid')).values(token=sa.bindparam('token'))
        conn.execute(stmt, updates)

    op.drop_index(op.f('ix_requests_token'))
    op.create_index(op.f('ix_requests_token'), 'requests', ['token'], unique=True)


def downgrade():
    pass



"""refactoring_members

Revision: 0021
Revision ID: d4628b068b5
Revises: 51ef836d17f
Create Date: 2015-08-21 14:16:56.533919

"""

# revision identifiers, used by Alembic.
revision = 'd4628b068b5'
down_revision = '51ef836d17f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('card_member_permissions',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('member_id', sa.Integer(), nullable=True),
    sa.Column('card_id', sa.Integer(), nullable=True),
    sa.Column('permission', sa.Integer(), nullable=True),
    sa.Column('created_by_id', sa.Integer(), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
    sa.ForeignKeyConstraint(['member_id'], ['card_members.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_member_permissions_card_id'), 'card_member_permissions', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_member_permissions_member_id'), 'card_member_permissions', ['member_id'], unique=False)
    op.create_index(op.f('ix_card_member_permissions_permission'), 'card_member_permissions', ['permission'], unique=False)
    op.add_column('card_members', sa.Column('email', sa.String(), nullable=True))
    op.create_index(op.f('ix_card_members_email'), 'card_members', ['email'], unique=False)
    op.drop_index('ix_card_members_permission', table_name='card_members')
    op.add_column('cards', sa.Column('members_cache_id', sa.String(), nullable=True))
    op.add_column('organization_members', sa.Column('email', sa.String(), nullable=True))
    op.create_index(op.f('ix_organization_members_email'), 'organization_members', ['email'], unique=False)
    op.add_column('users', sa.Column('perm_cache_id', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('users', 'perm_cache_id')
    op.drop_index(op.f('ix_organization_members_email'), table_name='organization_members')
    op.drop_column('organization_members', 'email')
    op.drop_column('cards', 'members_cache_id')
    op.create_index('ix_card_members_permission', 'card_members', ['permission'], unique=False)
    op.drop_index(op.f('ix_card_members_email'), table_name='card_members')
    op.drop_column('card_members', 'email')
    op.drop_index(op.f('ix_card_member_permissions_permission'), table_name='card_member_permissions')
    op.drop_index(op.f('ix_card_member_permissions_member_id'), table_name='card_member_permissions')
    op.drop_index(op.f('ix_card_member_permissions_card_id'), table_name='card_member_permissions')
    op.drop_table('card_member_permissions')
    ### end Alembic commands ###



"""email_templates

Revision: 0163
Revision ID: d8e7e0bbb09
Revises: 41184106cb68
Create Date: 2019-03-21 14:41:02.259607

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'd8e7e0bbb09'
down_revision = '41184106cb68'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('email_templates',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('owner_id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('subject', sa.String(), nullable=True),
        sa.Column('body', sa.Text(), nullable=True),
        sa.Column('sender', sa.String(), nullable=True),
        sa.Column('attachments_from_attributes', postgresql.ARRAY(sa.String), nullable=True),
        sa.Column('render_error', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_email_templates_project_id'), 'email_templates', ['project_id'], unique=False)

    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'MANAGE_EMAIL_TEMPLATES')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin = 'ADMIN'
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"MANAGE_EMAIL_TEMPLATES"}'::varchar[])
    """)

    op.execute("""
        UPDATE roles SET cache_id = uuid_generate_v4()
        WHERE roles.id IN (
            SELECT DISTINCT roles.id
            FROM roles, role_permissions
            WHERE role_permissions.role_id = roles.id
            AND roles.builtin = 'ADMIN'
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"MANAGE_EMAIL_TEMPLATES"}'::varchar[])
        )
    """)

    op.execute("""
        UPDATE projects SET members_cache_id = uuid_generate_v4()
    """)


def downgrade():
    op.drop_table('email_templates')



"""missing_foreign_key_constraint

Revision: 0009
Revision ID: 3a5f18e53496
Revises: 2075f8467bf0
Create Date: 2015-06-17 15:21:27.987975

"""

# revision identifiers, used by Alembic.
revision = '3a5f18e53496'
down_revision = '2075f8467bf0'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_index(op.f('ix_users_username'), 'users', ['username'], unique=False)
    op.drop_constraint('board_templates_owner_id_fkey', 'board_templates')
    op.create_foreign_key(None, 'board_templates', 'organizations', ['owner_id'], ['id'], ondelete='CASCADE')


def downgrade():
    op.drop_index(op.f('ix_users_username'), table_name='users')



"""views_lane_grouped_by_name

Revision: 0176
Revision ID: 4fc19fa2e6da
Revises: 4d90b902c3e0
Create Date: 2020-01-27 18:18:33.800145

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '4fc19fa2e6da'
down_revision = '4d90b902c3e0'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('views', sa.Column('lane_group_by_name', sa.String(), nullable=True))


def downgrade():
    op.drop_column('views', 'lane_group_by_name')



"""fix_duplicate_groups

Revision: 0115
Revision ID: 28ef45c94dfd
Revises: 4b8f6f8c1e02
Create Date: 2018-03-09 16:10:35.897709

"""

# revision identifiers, used by Alembic.
revision = '28ef45c94dfd'
down_revision = '4b8f6f8c1e02'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
)


def upgrade():
    conn = op.get_bind()

    q_groups = sa.text("""
        SELECT g1.id, count(gc1.card_id), g2.id, count(gc2.card_id)
        FROM card_groups g1
        JOIN card_groups g2
        ON g1.title = g2.title
        AND g1.group_type_id = g2.group_type_id
        AND g1.parent_id = g2.parent_id
        AND g1.id != g2.id
        LEFT JOIN card_groups_cards gc1
        ON gc1.group_id = g1.id
        LEFT JOIN card_groups_cards gc2
        ON gc2.group_id = g2.id
        GROUP BY g1.id, g2.id
        ORDER BY g1.id
    """)

    groups_to_delete = []
    for group in conn.execute(q_groups):
        if group[0] in groups_to_delete or group[2] in groups_to_delete:
            continue

        if group[1] == 0 and group[3] == 0 or group[1] > 0 and group[3] == 0:
            groups_to_delete.append(group[2])
        elif group[1] == 0 and group[3] > 0:
            groups_to_delete.append(group[0])

    conn.execute(groups.delete().where(groups.c.id.in_(groups_to_delete)))


def downgrade():
    pass



"""update_board_tpl

Revision: 0014
Revision ID: 5a9e03d271b2
Revises: 3ce3a91bc9eb
Create Date: 2015-07-07 14:35:46.443980

"""

# revision identifiers, used by Alembic.
revision = '5a9e03d271b2'
down_revision = '3ce3a91bc9eb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('board_template_lists', sa.Column('description', sa.Text(), nullable=True))
    op.add_column('board_template_lists', sa.Column('icon', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('board_template_lists', 'icon')
    op.drop_column('board_template_lists', 'description')
    ### end Alembic commands ###



"""card_groups

Revision: 0029
Revision ID: 25e88bf7445c
Revises: 43ed95babe0a
Create Date: 2015-10-26 20:28:04.796807

"""

# revision identifiers, used by Alembic.
revision = '25e88bf7445c'
down_revision = '43ed95babe0a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('card_group_types',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('scope', sa.Enum('card', 'project', 'org', name='card_group_type_scope'), nullable=False),
        sa.Column('one_group_per_card', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_group_types_owner_id'), 'card_group_types', ['owner_id'], unique=False)
    op.create_index(op.f('ix_card_group_types_scope'), 'card_group_types', ['scope'], unique=False)

    op.create_table('card_groups',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('group_type_id', sa.Integer(), nullable=True),
        sa.Column('parent_id', sa.Integer(), nullable=True),
        sa.Column('position', sa.Integer(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('is_archived', sa.Boolean(), nullable=True),
        sa.Column('archived_at', sa.DateTime(), nullable=True),
        sa.Column('meta', sa.PickleType(), nullable=True),
        sa.Column('color', sa.String(), nullable=True),
        sa.Column('icon', sa.String(), nullable=True),
        sa.Column('show_flow_indicator', sa.Boolean(), nullable=True),
        sa.Column('card_state', sa.String(), nullable=True),
        sa.Column('default_card_model_id', sa.Integer(), nullable=True),
        sa.Column('auto_assign_user', sa.String(), nullable=True),
        sa.Column('max_cards_limit', sa.Integer(), nullable=True),
        sa.Column('disallow_direct_card_creation', sa.Boolean(), nullable=True),
        sa.Column('start_date', sa.Date(), nullable=True),
        sa.Column('end_date', sa.Date(), nullable=True),
        sa.Column('milestone_id', sa.Integer(), nullable=True), #temporary
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['default_card_model_id'], ['card_models.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['group_type_id'], ['card_group_types.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['parent_id'], ['cards.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_groups_archived_at'), 'card_groups', ['archived_at'], unique=False)
    op.create_index(op.f('ix_card_groups_group_type_id'), 'card_groups', ['group_type_id'], unique=False)
    op.create_index(op.f('ix_card_groups_is_archived'), 'card_groups', ['is_archived'], unique=False)
    op.create_index(op.f('ix_card_groups_parent_id'), 'card_groups', ['parent_id'], unique=False)
    op.create_index(op.f('ix_card_groups_position'), 'card_groups', ['position'], unique=False)

    op.create_table('card_groups_cards',
        sa.Column('group_id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=False),
        sa.Column('position', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['group_id'], ['card_groups.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('group_id', 'card_id')
    )
    op.create_index(op.f('ix_card_groups_cards_position'), 'card_groups_cards', ['position'], unique=False)

    op.add_column('card_change_history', sa.Column('group_id', sa.Integer(), nullable=True))
    op.add_column('card_change_history', sa.Column('group_title', sa.String(), nullable=True))
    op.create_index(op.f('ix_card_change_history_group_id'), 'card_change_history', ['group_id'], unique=False)
    op.create_foreign_key(None, 'card_change_history', 'card_groups', ['group_id'], ['id'], ondelete='SET NULL')

    op.add_column('cards', sa.Column('archive_because_of_group_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'cards', 'card_groups', ['archive_because_of_group_id'], ['id'], ondelete='SET NULL')

    op.add_column('organizations', sa.Column('default_card_group_type_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'organizations', 'card_group_types', ['default_card_group_type_id'], ['id'], ondelete='SET NULL')

    op.drop_index('ix_card_change_history_list_id', table_name='card_change_history')
    op.drop_constraint('card_change_history_list_id_fkey', 'card_change_history', type_='foreignkey')
    op.drop_constraint('card_models_board_template_id_fkey', 'card_models', type_='foreignkey')
    op.drop_index('ix_cards_done', table_name='cards')
    op.drop_index('ix_cards_done_at', table_name='cards')
    op.drop_index('ix_cards_is_list_archived', table_name='cards')
    op.drop_index('ix_cards_list_id', table_name='cards')
    op.drop_index('ix_cards_list_position', table_name='cards')
    op.drop_index('ix_cards_milestone_id', table_name='cards')
    op.drop_index('ix_cards_reopened', table_name='cards')
    op.drop_constraint('cards_list_id_fkey', 'cards', type_='foreignkey')
    op.drop_constraint('cards_milestone_id_fkey', 'cards', type_='foreignkey')

    op.execute("""
INSERT INTO card_group_types
    (created_at, owner_id, name, scope, one_group_per_card)
SELECT current_timestamp, id, 'Lists', 'card', true FROM organizations;

UPDATE organizations SET default_card_group_type_id = (SELECT id FROM card_group_types WHERE owner_id = organizations.id);

INSERT INTO card_group_types
    (created_at, owner_id, name, scope, one_group_per_card)
SELECT current_timestamp, id, 'Milestones', 'project', true FROM organizations;


INSERT INTO card_groups
    (id, created_at, updated_at, created_by_id, group_type_id, parent_id, position, title,
     description, is_archived, archived_at, meta, color, icon, show_flow_indicator, card_state,
     default_card_model_id, auto_assign_user, max_cards_limit, disallow_direct_card_creation)
SELECT cl.id, cl.created_at, cl.updated_at, cl.created_by_id, (
        SELECT id FROM card_group_types WHERE owner_id = c.owner_id AND name = 'Lists'
     ), cl.parent_id, cl.position, cl.title,
     cl.description, cl.is_archived, cl.archived_at, cl.meta, cl.bgcolor, cl.icon, cl.show_flow_indicator, cl.card_state,
     cl.default_card_model_id, cl.auto_assign_user, cl.max_cards_limit, cl.disallow_direct_card_creation
FROM card_lists cl JOIN cards c ON c.id = cl.parent_id;

SELECT setval('card_groups_id_seq', nextval('card_lists_id_seq'));

INSERT INTO card_groups_cards
    SELECT c.list_id, c.id, c.list_position FROM cards c WHERE c.list_id is not null;


UPDATE card_change_history SET group_id = list_id, group_title = list_title;


INSERT INTO card_groups
    (milestone_id, created_at, updated_at, created_by_id, group_type_id, parent_id, title,
     is_archived, color, start_date, end_date)
SELECT m.id, m.created_at, m.updated_at, m.created_by_id, (
        SELECT id FROM card_group_types WHERE owner_id = c.owner_id AND name = 'Milestones'
     ), m.parent_id, m.title, m.closed, m.color, m.start_date, m.end_date
FROM milestones m JOIN cards c ON c.id = m.parent_id;

INSERT INTO card_groups_cards (group_id, card_id)
    SELECT cg.id, c.id
    FROM cards c JOIN card_groups cg ON cg.milestone_id = c.milestone_id;

WITH positions as (
    SELECT group_id, card_id, row_number() OVER (partition by group_id order by card_id) AS p
    FROM card_groups_cards WHERE position is null
) UPDATE card_groups_cards set position = positions.p - 1 FROM positions
WHERE card_groups_cards.position is null AND positions.group_id = card_groups_cards.group_id
AND positions.card_id = card_groups_cards.card_id;

WITH positions as (
    SELECT group_type_id, id, row_number() OVER (partition by group_type_id order by id) AS p
    FROM card_groups WHERE position is null
) UPDATE card_groups set position = positions.p - 1 FROM positions
WHERE card_groups.position is null AND positions.group_type_id = card_groups.group_type_id
AND positions.id = card_groups.id;
""")


def downgrade():
    op.drop_column('organizations', 'default_card_group_type_id')

    op.create_foreign_key('cards_milestone_id_fkey', 'cards', 'milestones', ['milestone_id'], ['id'])
    op.create_foreign_key('cards_list_id_fkey', 'cards', 'card_lists', ['list_id'], ['id'])
    op.drop_column('cards', 'archive_because_of_group_id')
    op.create_index('ix_cards_reopened', 'cards', ['reopened'], unique=False)
    op.create_index('ix_cards_milestone_id', 'cards', ['milestone_id'], unique=False)
    op.create_index('ix_cards_list_position', 'cards', ['list_position'], unique=False)
    op.create_index('ix_cards_list_id', 'cards', ['list_id'], unique=False)
    op.create_index('ix_cards_is_list_archived', 'cards', ['is_list_archived'], unique=False)
    op.create_index('ix_cards_done_at', 'cards', ['done_at'], unique=False)
    op.create_index('ix_cards_done', 'cards', ['done'], unique=False)

    op.create_foreign_key('card_models_board_template_id_fkey', 'card_models', 'board_templates', ['board_template_id'], ['id'])

    op.create_foreign_key('card_change_history_list_id_fkey', 'card_change_history', 'card_lists', ['list_id'], ['id'])
    op.create_index('ix_card_change_history_list_id', 'card_change_history', ['list_id'], unique=False)
    op.drop_index(op.f('ix_card_change_history_group_id'), table_name='card_change_history')
    op.drop_column('card_change_history', 'group_title')
    op.drop_column('card_change_history', 'group_id')

    op.drop_index(op.f('ix_card_groups_cards_position'), table_name='card_groups_cards')
    op.drop_table('card_groups_cards')

    op.drop_index(op.f('ix_card_groups_position'), table_name='card_groups')
    op.drop_index(op.f('ix_card_groups_parent_id'), table_name='card_groups')
    op.drop_index(op.f('ix_card_groups_is_archived'), table_name='card_groups')
    op.drop_index(op.f('ix_card_groups_group_type_id'), table_name='card_groups')
    op.drop_index(op.f('ix_card_groups_archived_at'), table_name='card_groups')
    op.drop_table('card_groups')

    op.drop_index(op.f('ix_card_group_types_scope'), table_name='card_group_types')
    op.drop_index(op.f('ix_card_group_types_owner_id'), table_name='card_group_types')
    op.drop_table('card_group_types')



"""users_enable_emails

Revision: 0118
Revision ID: 252587fbe66d
Revises: 4c92555fa6cb
Create Date: 2018-03-14 11:04:14.326443

"""

# revision identifiers, used by Alembic.
revision = '252587fbe66d'
down_revision = '4c92555fa6cb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('enable_emails', sa.Boolean(), nullable=True))
    op.execute("UPDATE users SET enable_emails = enable_email_notifications")


def downgrade():
    op.drop_column('users', 'enable_emails')



"""project_gantt_config

Revision ID: 0279
Revises: 0278
Create Date: 2024-11-18 22:34:16.800414

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
from sqlalchemy.ext.mutable import MutableDict


# revision identifiers, used by Alembic.
revision = '0279'
down_revision = '0278'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('gantt_config', MutableDict.as_mutable(postgresql.JSONB), nullable=True))


def downgrade():
    op.drop_column('projects', 'gantt_config')



"""fix_parents_ids

Revision: 0229
Revision ID: 9a7f12dce0ad
Revises: ad80214e4b12
Create Date: 2022-02-25 16:22:10.965530

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '9a7f12dce0ad'
down_revision = 'ad80214e4b12'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH
        RECURSIVE recompute AS
        (
            -- Anchor
            SELECT id "origin_id", 1 "level", parent_id, array[parent_id] "pids"
            FROM cards
            WHERE parent_id IS NOT NULL

            UNION ALL

            -- Recursive Member
            SELECT r.origin_id, r.level+1, c.parent_id, array[c.parent_id]||r.pids
            FROM cards c
            JOIN recompute r ON c.id = r.parent_id
            WHERE c.parent_id IS NOT NULL
        ),

        levels AS
        (
            SELECT origin_id, max(level) "max"
            FROM recompute
            GROUP BY origin_id
        ),

        new_parents AS
        (
            SELECT r.origin_id "id", r.pids
            FROM recompute r
            JOIN levels l ON l.origin_id = r.origin_id
            WHERE r.level = l.max
        )

        UPDATE cards c
        SET parents_ids = np.pids
        FROM new_parents np
        WHERE np.id = c.id
        AND np.pids != c.parents_ids;
    """)


def downgrade():
    pass



"""form_unlisted

Revision: 0239
Revision ID: 3fa23c3f6c07
Revises: f9352a1f06eb
Create Date: 2022-03-15 15:51:03.480390

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3fa23c3f6c07'
down_revision = 'f9352a1f06eb'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('request_forms', sa.Column('unlisted', sa.Boolean(), nullable=True))
    op.execute("UPDATE request_forms SET unlisted = false")


def downgrade():
    op.drop_column('request_forms', 'unlisted')



"""form_fields

Revision: 0089
Revision ID: 6dac66a4490
Revises: 57c60056d090
Create Date: 2017-03-16 11:50:20.360600

"""

# revision identifiers, used by Alembic.
revision = '6dac66a4490'    
down_revision = '57c60056d090'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('request_form_fields',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('request_form_id', sa.Integer(), nullable=True),
        sa.Column('label', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('required', sa.Boolean(), nullable=True),
        sa.Column('attribute_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['request_form_id'], ['request_forms.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['attribute_id'], ['attributes.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_request_form_fields_request_form_id'), 'request_form_fields', ['request_form_id'], unique=False)

    op.execute("""
        INSERT INTO request_form_fields (request_form_id, label, required)
        SELECT id, coalesce(title_label, 'Title'), true FROM request_forms WHERE ask_title
    """)

    op.execute("""
        INSERT INTO request_form_fields (request_form_id, label, required, attribute_id)
        SELECT request_form_id, attributes.name, false, attribute_id
        FROM request_form_attributes JOIN attributes ON attributes.id = attribute_id ORDER BY id
    """)

    op.drop_column('request_forms', 'ask_title')
    op.drop_column('request_forms', 'title_label')
    op.drop_table('request_form_attributes')


def downgrade():
    pass



"""ai_context

Revision ID: 0267
Revises: 0266
Create Date: 2025-06-17 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0267'
down_revision = '0266'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('ai_context', sa.String()))


def downgrade():
    op.drop_column('projects', 'ai_context')


"""delete_projects_without_top_card

Revision: 0134
Revision ID: 11fab9822bf6
Revises: 4cee323fc652
Create Date: 2018-06-12 15:45:37.410916

"""

# revision identifiers, used by Alembic.
revision = '11fab9822bf6'
down_revision = '4cee323fc652'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute('DELETE FROM projects WHERE top_level_card_id IS NULL')


def downgrade():
    pass



"""card_files

Revision: 0038
Revision ID: 448178393554
Revises: 11af45c6c79e
Create Date: 2015-11-28 18:02:05.528899

"""

# revision identifiers, used by Alembic.
revision = '448178393554'
down_revision = '11af45c6c79e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

card_attributes_table = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('type_name', sa.String),
    sa.Column('value', postgresql.JSONB)
)

def upgrade():
    card_files = op.create_table('card_files',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('filename', sa.String(), nullable=True),
        sa.Column('mimetype', sa.String(), nullable=True),
        sa.Column('size', sa.Integer(), nullable=True),
        sa.Column('is_external', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_files_created_at'), 'card_files', ['created_at'], unique=False)

    cards_card_files = op.create_table('cards_card_files',
        sa.Column('file_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['file_id'], ['card_files.id'], ondelete='CASCADE')
    )

    conn = op.get_bind()
    for attr in conn.execute(card_attributes_table.select().where(card_attributes_table.c.type_name == 'files')):
        if not attr.value:
            continue
        files = []
        for f in attr.value:
            r = conn.execute(card_files.insert().values(
                name=f['name'],
                mimetype=f.get('mimetype'),
                filename=f.get('filename', f.get('s3filename', f.get('url'))),
                is_external='url' not in f
            ))
            files.append(r.inserted_primary_key[0])
        conn.execute(card_attributes_table.update()
            .where(card_attributes_table.c.id==attr.id).values(value=files))
        if files:
            op.bulk_insert(cards_card_files,
                [{"card_id":  attr.card_id, "file_id": id} for id in files])



def downgrade():
    op.drop_table('cards_card_files')
    op.drop_index(op.f('ix_card_files_created_at'), table_name='card_files')
    op.drop_table('card_files')


"""ai_provider

Revision ID: 0272
Revises: 0271
Create Date: 2025-06-17 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0272'
down_revision = '0271'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('ai_provider', sa.Text()))
    op.add_column('organizations', sa.Column('ai_context', sa.Text()))

def downgrade():
    op.drop_column('organizations', 'ai_provider')
    op.drop_column('organizations', 'ai_context')


"""email_templates_counter

Revision: 0166
Revision ID: 46d38eb71ef6
Revises: f5482137635
Create Date: 2019-06-20 13:58:29.594346

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '46d38eb71ef6'
down_revision = 'f5482137635'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('email_templates', sa.Column('counter', sa.Integer(), nullable=True))
    op.add_column('email_templates', sa.Column('last_sent_at', sa.DateTime(), nullable=True))
    op.add_column('organizations', sa.Column('email_templates_send_counter', sa.Integer(), nullable=True))
    op.execute("UPDATE email_templates SET counter = 0")
    op.execute("UPDATE organizations SET email_templates_send_counter = 0")


def downgrade():
    op.drop_column('email_templates', 'counter')
    op.drop_column('email_templates', 'last_sent_at')
    op.drop_column('organizations', 'email_templates_counter')



"""activities

Revision: 0062
Revision ID: 3ab6d84b6186
Revises: bfc8eb1f8c2
Create Date: 2016-06-21 16:53:01.468562

"""

# revision identifiers, used by Alembic.
revision = '3ab6d84b6186'
down_revision = 'bfc8eb1f8c2'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


activities_table = sa.Table(
    'activities',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('subject_value', sa.PickleType()),
    sa.Column('sub_object_value', postgresql.JSON()),
)


def upgrade():
    op.execute("DROP VIEW IF EXISTS metrics_project_active_months");
    op.execute("DROP VIEW IF EXISTS metrics_user_active_months");

    op.create_table('card_comments',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('message', sa.Text(), nullable=True),
        sa.Column('is_pinned', sa.Boolean(), nullable=True),
        sa.Column('stream_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_comments_card_id'), 'card_comments', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_comments_created_by_id'), 'card_comments', ['created_by_id'], unique=False)

    op.execute("""INSERT INTO card_comments (card_id, created_by_id, created_at, message, is_pinned, stream_id)
        SELECT card_id, created_by_id, created_at, message, is_pinned, id FROM stream WHERE item_type = 'comment'""")

    op.rename_table('stream', 'activities')
    op.add_column('activities', sa.Column('owner_id', sa.Integer(), nullable=True))
    op.add_column('activities', sa.Column('verb', sa.String(), nullable=True))
    op.add_column('activities', sa.Column('object_type', sa.String(), index=True))
    op.add_column('activities', sa.Column('object_id', sa.Integer(), index=True))
    op.add_column('activities', sa.Column('object_value', postgresql.JSON()))
    op.add_column('activities', sa.Column('sub_object_type', sa.String(), index=True))
    op.add_column('activities', sa.Column('sub_object_id', sa.Integer(), index=True))
    op.add_column('activities', sa.Column('sub_object_value', postgresql.JSON()))
    op.add_column('activities', sa.Column('origin_type', sa.String(), index=True))
    op.add_column('activities', sa.Column('origin_id', sa.Integer(), index=True))
    op.add_column('activities', sa.Column('origin_value', postgresql.JSON()))
    op.add_column('activities', sa.Column('target_type', sa.String(), index=True))
    op.add_column('activities', sa.Column('target_id', sa.Integer(), index=True))
    op.add_column('activities', sa.Column('target_value', postgresql.JSON()))

    op.create_foreign_key(None, 'activities', 'organizations', ['owner_id'], ['id'], ondelete='CASCADE')

    conn = op.get_bind()
    for item in conn.execute(activities_table.select().where(activities_table.c.subject_value!=None)):
        conn.execute(activities_table.update().where(activities_table.c.id==item.id).values(sub_object_value=item.subject_value))

    op.execute("""UPDATE activities SET verb = upper(label), sub_object_type = subject_type, sub_object_id = subject_id,
        owner_id = (SELECT owner_id FROM cards WHERE id = card_id), object_type='Card', object_id=card_id,
        object_value = (SELECT to_json(title) FROM cards WHERE id = card_id)""")

    op.execute("""UPDATE activities SET verb = 'CREATE', target_type = 'Card', target_id = object_id, target_value = object_value,
        object_type = 'CardComment', object_id = (SELECT id FROM card_comments WHERE stream_id = activities.id), object_value = to_json(message),
        message = '{actor} left a comment on {target}',
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE item_type = 'comment'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} attached {sub_object} to {target}',
        sub_object_type = 'CardFile', target_type = 'Card', target_id = object_id, target_value = object_value,
        object_type = 'CardModelAttribute', object_id = null, object_value = to_json('attachments'::text) WHERE label = 'file'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} assigned {sub_object} to {target}',
        sub_object_type = 'User', target_type = 'Card', target_id = object_id, target_value = object_value,
        object_type = 'CardModelAttribute', object_id = null, object_value = to_json('assignees'::text) WHERE label = 'assigned'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} unassigned {sub_object} from {target}',
        sub_object_type = 'User', target_type = 'Card', target_id = object_id, target_value = object_value,
        object_type = 'CardModelAttribute', object_id = null, object_value = to_json('assignees'::text) WHERE label = 'unassigned'""")

    op.execute("""UPDATE activities SET verb = 'ADD', message = replace(replace(message, '{subject}', '{object}'), '{user}', '{actor}'),
        target_type = 'Card', target_id = object_id, target_value = object_value,
        object_type = 'commit', object_id = null, object_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'commit'""")

    op.execute("""UPDATE activities SET verb = 'JOIN', message = '{actor} joined {object}' WHERE label = 'joined'""")

    op.execute("""UPDATE activities SET verb = 'LEFT', message = '{actor} left {object}'
        WHERE label = 'left' AND message = '{user} has left'""")

    op.execute("""UPDATE activities SET verb = 'LEFT', message = '{actor} removed {sub_object} from members of {object}'
        WHERE label = 'left' AND message = '{user} removed {subject} from members'""")

    op.execute("""UPDATE activities SET verb = 'CREATE', message = '{actor} created {object} under {target}',
        target_type = 'Card', target_id = object_id, target_value = object_value,
        object_type = 'CardGroup', object_id = sub_object_id, object_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'group_created'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} changed the title from {origin}',
        origin_value = sub_object_value, sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'updated' and message = '{user} changed the title from {subject}'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} changed the background color',
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'updated' and message = '{user} changed the background color'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} changed the state of {object} to {target}',
        target_value = sub_object_value, sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'updated' and message = '{user} changed the state of the card to {subject}'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} switched the model of {object} to {target}',
        target_type = sub_object_type, target_id = sub_object_id, target_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'updated' and message = '{user} switched the model to {subject}'""")

    op.execute("""UPDATE activities SET verb = 'MOVE', message = '{actor} moved {object} to {target}',
        target_type = sub_object_type, target_id = sub_object_id, target_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'moved' and message = '{user} moved {target} to {subject}'""")

    op.execute("""UPDATE activities SET verb = 'MOVE', message = '{actor} moved {object} to a board under a different project'
        WHERE label = 'moved' and message = '{user} moved {target} to a board under a different project'""")

    op.execute("""UPDATE activities SET verb = 'COPY', message = '{actor} created {object} as a copy'
        WHERE label = 'copied' and message = '{user} copied {target} to {subject}'""")

    op.execute("""UPDATE activities SET verb = 'MOVE', message = '{actor} converted {object} to a project'
        WHERE label = 'moved' and message = '{user} converted {target} to a project'""")

    op.execute("""UPDATE activities SET verb = 'ARCHIVE', message = '{actor} archived {object}'
        WHERE label = 'archived' and message = '{user} archived {target}'""")

    op.execute("""UPDATE activities SET verb = 'RESTORE', message = '{actor} restored {object}'
        WHERE label = 'restored'""")

    op.execute("""UPDATE activities SET verb = 'CREATE', message = '{actor} created {object}'
        WHERE label = 'created' and message = '{user} added {target}'""")

    op.execute("""DELETE FROM activities WHERE label = 'created' and message = '{user} linked to {subject}'""")
    op.execute("""DELETE FROM activities WHERE label = 'created' and message = '{user} created {subject}' and sub_object_type = 'Card'""")

    op.execute("""UPDATE activities SET verb = 'UPDATE', message = '{actor} changed the background color of {object}',
        object_type = 'CardGroup', object_id = sub_object_id, object_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'group_updated' and message = '{user} changed the background color of group {subject}'""")

    op.execute("""DELETE FROM activities WHERE label = 'group_updated' and message = '{user} renamed group {title} to {subject}'""")

    op.execute("""UPDATE activities SET verb = 'ARCHIVE', message = '{actor} archived {object}',
        object_type = 'CardGroup', object_id = sub_object_id, object_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'group_archived'""")

    op.execute("""UPDATE activities SET verb = 'RESTORE', message = '{actor} restored {object}',
        object_type = 'CardGroup', object_id = sub_object_id, object_value = sub_object_value,
        sub_object_type = null, sub_object_id = null, sub_object_value = null
        WHERE label = 'group_restored'""")

    op.execute("""UPDATE activities SET message = replace(replace(replace(message, '{target}', '{object}'), '{subject}', '{sub_object}'), '{user}', '{actor}')
        WHERE message like '{user}%'""")
    op.execute("UPDATE activities SET verb = 'CREATE' WHERE verb = 'CREATED'")
    op.execute("UPDATE activities SET verb = 'UPDATE' WHERE verb = 'UPDATED'")
    op.execute("UPDATE activities SET verb = 'COPY' WHERE verb = 'COPIED'")
    op.execute("UPDATE activities SET verb = 'MOVE' WHERE verb = 'MOVED'")
    op.execute("UPDATE activities SET verb = 'RESTORE' WHERE verb = 'RESTORED'")
    op.execute("UPDATE activities SET verb = 'CREATE' WHERE verb = 'LIST_CREATED'")
    op.execute("UPDATE activities SET verb = 'UPDATE' WHERE verb = 'LIST_UPDATED'")
    op.execute("UPDATE activities SET verb = 'ARCHIVE' WHERE verb = 'LIST_ARCHIVED'")
    op.execute("UPDATE activities SET verb = 'RESTORE' WHERE verb = 'LIST_RESTORED'")
    op.execute("UPDATE activities SET message = '{actor} created {sub_object}' WHERE message = '{actor} created {sub_object} in list {list}'")
    op.execute("DELETE FROM activities WHERE message = '{actor} renamed list {title} to {sub_object}'")

    op.create_index(op.f('ix_activities_owner_id'), 'activities', ['owner_id'], unique=False)
    op.create_index(op.f('ix_activities_created_at'), 'activities', ['created_at'], unique=False)

    op.drop_constraint('stream_card_id_fkey', 'activities')
    op.drop_index(op.f('ix_stream_card_id'), table_name='activities')
    op.drop_index(op.f('ix_stream_is_pinned'), table_name='activities')
    op.drop_index(op.f('ix_stream_item_type'), table_name='activities')
    op.drop_column('activities', 'subject_type')
    op.drop_column('activities', 'subject_id')
    op.drop_column('activities', 'subject_value')
    op.drop_column('activities', 'args')
    op.drop_column('activities', 'label')
    op.drop_column('activities', 'card_id')
    op.drop_column('activities', 'item_type')
    op.drop_column('activities', 'is_pinned')

    op.drop_column('card_comments', 'stream_id')


def downgrade():
    pass



"""templates_media_user_preferred_view

Revision: 0192
Revision ID: b01140bf6710
Revises: 16a37635c0ed
Create Date: 2020-09-02 13:54:23.709035

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'b01140bf6710'
down_revision = '16a37635c0ed'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('preferred_default_view', sa.String(), nullable=True))
    op.add_column('projects', sa.Column('template_preview_media_id', sa.Integer(), nullable=True))
    op.execute("UPDATE users SET preferred_default_view = 'kanban'")


def downgrade():
    op.drop_column('users', 'preferred_default_view')
    op.drop_column('projects', 'template_preview_media_id')



"""refactoring

Revision: 0156
Revision ID: 3dd85740e8ac
Revises: 284aac8ecbc6
Create Date: 2018-12-21 16:30:30.541069

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy import text
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3dd85740e8ac'
down_revision = '284aac8ecbc6'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('last_password_change_from', sa.String(), nullable=True))
    op.add_column('user_logins', sa.Column('login_country', sa.String(), nullable=True))
    op.add_column('user_logins', sa.Column('login_user_agent', sa.String(), nullable=True))

    op.create_table('user_oauth_tokens',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('provider', sa.String(), nullable=True),
        sa.Column('token_type', sa.String(), nullable=True),
        sa.Column('access_token', sa.String(), nullable=True),
        sa.Column('access_token_secret', sa.String(), nullable=True),
        sa.Column('refresh_token', sa.String(), nullable=True),
        sa.Column('token_expires_at', sa.DateTime(), nullable=True),
        sa.Column('scope', sa.String(), nullable=True),
        sa.Column('data', postgresql.JSONB(), nullable=True),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_index(op.f('ix_user_oauth_tokens_user_id_provider'), 'user_oauth_tokens', ['user_id', 'provider'], unique=False)
    op.create_index(op.f('ix_user_oauth_tokens_data'), 'user_oauth_tokens', ['data'], unique=False)
    op.create_index(op.f('ix_users_username_lower'), 'users', [text('lower(username)')], unique=False)

    op.execute("""
        INSERT INTO user_oauth_tokens (user_id, created_at, provider, access_token, access_token_secret, data)
        SELECT id, now(), 'trello', trello_oauth_token, trello_oauth_token_secret, ('{"user_id": "' || trello_user_id || '", "username": "' || trello_username || '"}')::jsonb
        FROM users WHERE ARRAY['trello'] <@ auth_providers::text[]
    """)

    op.execute("""
        INSERT INTO user_oauth_tokens (user_id, created_at, provider, access_token, scope, data)
        SELECT id, now(), 'github', github_access_token, github_scope, ('{"id": "' || github_id || '", "username": "' || github_username || '", "email": "' || github_email || '"}')::jsonb
        FROM users WHERE ARRAY['github'] <@ auth_providers::text[]
    """)

    op.execute("""
        INSERT INTO user_oauth_tokens (user_id, created_at, provider, access_token, data)
        SELECT id, now(), 'asana', asana_token, ('{"id": "' || asana_id || '", "name": "' || asana_name || '", "email": "' || asana_email || '"}')::jsonb
        FROM users WHERE ARRAY['asana'] <@ auth_providers::text[]
    """)

    op.execute("""
        INSERT INTO user_oauth_tokens (user_id, created_at, provider, access_token, refresh_token, token_expires_at, data)
        SELECT id, now(), 'google', google_token, google_refresh_token, google_token_expiry, ('{"id": "' || google_id || '", "name": "' || google_name || '", "email": "' || google_email || '"}')::jsonb
        FROM users WHERE ARRAY['google'] <@ auth_providers::text[]
    """)


def downgrade():
    op.drop_column('users', 'last_password_change_from')
    op.drop_column('user_logins', 'login_country')
    op.drop_column('user_logins', 'login_user_agent')
    op.drop_table('user_oauth_tokens')



"""signup_referrer

Revision: 0202
Revision ID: 47e876a62c34
Revises: d8b702a1eee2
Create Date: 2021-01-15 16:18:48.661982

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '47e876a62c34'
down_revision = 'd8b702a1eee2'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('signup_referrer', sa.String(), nullable=True))


def downgrade():
    op.drop_column('users', 'signup_referrer')



"""oauth_auto_app

Revision ID: 0245
Revises: 3dc5ebfeecea
Create Date: 2022-06-03 10:33:29.708025

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0245'
down_revision = '3dc5ebfeecea'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('oauth_apps', sa.Column('auto_app_name', sa.String()))


def downgrade():
    op.drop_column('oauth_apps', 'auto_app_name')



"""enabled_group_types

Revision: 0035
Revision ID: 30849ae581ab
Revises: 166735e0e8f6
Create Date: 2015-11-22 21:11:04.776843

"""

# revision identifiers, used by Alembic.
revision = '30849ae581ab'
down_revision = '2a46d7b96e63'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('card_enabled_group_types',
        sa.Column('group_type_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['group_type_id'], ['card_group_types.id'], ondelete='CASCADE')
    )

    op.add_column('card_group_types', sa.Column('auto_enabled_on_new_projects', sa.Boolean(), nullable=True))
    op.add_column('card_group_types', sa.Column('is_project_specific', sa.Boolean(), nullable=True))
    
    op.add_column('cards', sa.Column('default_card_group_type_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'cards', 'card_group_types', ['default_card_group_type_id'], ['id'], ondelete='SET NULL')

    op.execute("UPDATE card_group_types SET is_project_specific = false, auto_enabled_on_new_projects = true")
    op.execute("""INSERT INTO card_enabled_group_types (group_type_id, card_id)
        SELECT cgt.id, c.id FROM card_group_types cgt JOIN organizations o ON o.id = cgt.owner_id
            JOIN cards c ON c.owner_id = o.id WHERE c.parent_id is null""")
    op.execute("""UPDATE cards SET default_card_group_type_id = o.default_card_group_type_id
        FROM organizations o WHERE o.id = cards.owner_id AND cards.parent_id IS NULL""")


def downgrade():
    op.drop_table('card_enabled_group_types')
    op.drop_column('cards', 'default_card_group_type_id')
    op.drop_column('card_group_types', 'is_project_specific')
    op.drop_column('card_group_types', 'auto_enabled_on_new_projects')


"""user_favorites

Revision: 0235
Revision ID: 8abfd8e00867
Revises: 249b8b1c2bd6
Create Date: 2022-02-10 16:30:40.929731

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '8abfd8e00867'
down_revision = '249b8b1c2bd6'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('user_favorites',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('object_type', sa.String(), nullable=False),
        sa.Column('object_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_user_favorites_unique'), 'user_favorites', ['user_id', 'object_type', 'object_id'], unique=True)

    op.execute("""
        INSERT INTO user_favorites (user_id, object_type, object_id)
        SELECT DISTINCT users_pins.user_id, 'project', cards.project_id
        FROM cards
        JOIN users_pins ON users_pins.card_id = cards.id
        WHERE cards.parent_id IS NULL
    """)

    op.execute("""
        INSERT INTO user_favorites (user_id, object_type, object_id)
        SELECT DISTINCT users_pins.user_id, 'card', cards.id
        FROM cards
        JOIN users_pins ON users_pins.card_id = cards.id
        WHERE cards.parent_id IS NOT NULL
    """)


def downgrade():
    op.drop_table('user_favorites')



"""labels color

Revision: 0042
Revision ID: da5397402d5
Revises: 5409e2aad044
Create Date: 2015-12-09 19:58:09.950353

"""

# revision identifiers, used by Alembic.
revision = 'da5397402d5'
down_revision = '5409e2aad044'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select
from sqlalchemy.dialects import postgresql
import pickle

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('children_attributes_ctx', sa.PickleType, default=dict)
)

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('type_name', sa.String),
    sa.Column('card_model_attribute_id', sa.Integer)
)

def upgrade():
    conn = op.get_bind()
    q = select([card_attributes, cards.c.parent_id])\
        .where(sa.and_(cards.c.id == card_attributes.c.card_id, card_attributes.c.type_name == 'labels'))\
        .order_by(cards.c.parent_id)

    parent_id = None
    ctx = None
    for attr in conn.execute(q):
        if parent_id != attr.parent_id:
            parent_id = attr.parent_id
            if parent_id is None:
                parent_id = attr.card_id
            ctx = conn.execute(select([cards.c.children_attributes_ctx]).where(cards.c.id==parent_id)).scalar()
        key = '%s:%s' % (attr.name, attr.type_name)
        if key in ctx:
            newKey = '%i:%s' % (attr.card_model_attribute_id, attr.type_name)
            ctx[newKey] = {}
            for title, color in ctx.get(key, {}).items():
                ctx[newKey][title] = color
            del ctx[key]
            conn.execute(cards.update().where(cards.c.id==parent_id).values(children_attributes_ctx=ctx))

def downgrade():
    pass



"""activity_user_id

Revision: 0213
Revision ID: 8db3ad545942
Revises: 44a1660b8ce7
Create Date: 2021-06-01 22:37:30.858194

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '8db3ad545942'
down_revision = '44a1660b8ce7'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('activities', sa.Column('user_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'activities', 'users', ['user_id'], ['id'], ondelete='CASCADE')
    op.create_index(None, 'activities', ['user_id'], unique=False)


def downgrade():
    op.drop_column('activities', 'user_id')



"""cards_title_trgm_index

Revision: 0097
Revision ID: 4e63e5c80ada
Revises: 5397fd6fbb4d
Create Date: 2017-06-20 17:01:58.608162

"""

# revision identifiers, used by Alembic.
revision = '4e63e5c80ada'
down_revision = '5397fd6fbb4d'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("CREATE EXTENSION IF NOT EXISTS pg_trgm;")
    op.execute("CREATE INDEX cards_title_trgm_idx ON cards USING gin(title gin_trgm_ops);")


def downgrade():
    op.execute("DROP INDEX cards_title_trgm_idx")



"""project_system_apps

Revision: 0223
Revision ID: 22b7db58107b
Revises: 0546e8ee2d53
Create Date: 2021-10-28 16:18:09.449514

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '22b7db58107b'
down_revision = '0546e8ee2d53'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('system_apps', postgresql.ARRAY(sa.String), nullable=True))
    op.execute("UPDATE projects SET system_apps = '{automations}'::text[]")


def downgrade():
    op.drop_column('projects', 'system_apps')



"""ai_settings

Revision ID: 0266
Revises: 0265
Create Date: 2025-06-17 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0266'
down_revision = '0265'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('ai_enabled', sa.Boolean()))
    op.add_column('organizations', sa.Column('ai_base_url', sa.String()))
    op.add_column('organizations', sa.Column('ai_api_key', sa.String()))
    op.add_column('organizations', sa.Column('ai_model', sa.String()))


def downgrade():
    op.drop_column('organizations', 'ai_enabled')
    op.drop_column('organizations', 'ai_base_url')
    op.drop_column('organizations', 'ai_api_key')
    op.drop_column('organizations', 'ai_model')